@@ -384,6 +384,14 @@ void KArtsModule::saveParams( void )
 	midiConfig->writePathEntry( "mapFilename", hardware->midiMapper->url() );
 
 	delete midiConfig;
+    
+    KConfig *knotifyConfig = new KConfig(  "knotifyrc", false );
+    
+    knotifyConfig->setGroup(  "StartProgress" );
+    knotifyConfig->writeEntry(  "Arts Init", startServer->isChecked() );
+    knotifyConfig->writeEntry(  "Use Arts", startServer->isChecked() );
+    
+    delete knotifyConfig;
 
 	config->sync();
 }
