@@ -1,4 +1,5 @@
 /* vi: ts=8 sts=4 sw=4
+ * kate: space-indent on; indent-width 4; indent-mode cstyle;
  *
  * This file is part of the KDE project, module kdesktop.
  * Copyright (C) 1999 Geert Jansen <g.t.jansen@stud.tue.nl>
@@ -13,6 +14,7 @@
 
 #include <qstringlist.h>
 #include <qcolor.h>
+#include <qvaluevector.h>
 
 template <class QString, class T> class QMap;
 class KStandardDirs;
@@ -149,13 +151,23 @@ class KBackgroundSettings
       public KBackgroundProgram
 {
 public:
-    KBackgroundSettings(int desk, KConfig *config);
+    /**
+     * @param drawBackgroundPerScreen if false, then all screens (in xinerama
+     * mode) will be treated as one big display, and the "screen" paramater
+     * will be ignored.
+     */
+    KBackgroundSettings(int desk, int screen, bool drawBackgroundPerScreen, KConfig *config);
     ~KBackgroundSettings();
 
     void copyConfig(const KBackgroundSettings*);
 
+    bool drawBackgroundPerScreen() const { return m_bDrawBackgroundPerScreen; }
+    void setDrawBackgroundPerScreen(bool draw);
+
     int desk() const { return m_Desk; }
-    void load(int desk, bool reparseConfig=true);
+    int screen() const { return m_Screen; }
+//     void load(int desk, int screen, bool drawBackgroundPerScreen, bool reparseConfig=true);
+    void load(int desk, int screen, bool drawBackgroundPerScreen, bool reparseConfig);
 
     void setColorA(const QColor &color);
     QColor colorA() const { return m_ColorA; }
@@ -194,7 +206,7 @@ public:
 
     enum WallpaperMode {
 	NoWallpaper, Centred, Tiled, CenterTiled, CentredMaxpect, TiledMaxpect,
-	Scaled, CentredAutoFit, lastWallpaperMode
+	Scaled, CentredAutoFit, ScaleAndCrop, lastWallpaperMode
     };
     void setWallpaperMode(int mode);
     int wallpaperMode() const { return m_WallpaperMode; }
@@ -226,7 +238,7 @@ public:
     void updateWallpaperFiles();
     void randomizeWallpaperFiles();
 
-    QString currentWallpaper();
+    QString currentWallpaper() const;
     /**
      * @return true if the currentWallpaper has changed
      */
@@ -236,19 +248,20 @@ public:
 
     void readSettings(bool reparse=false);
     void writeSettings();
+    QString configGroupName() const;
 
     int hash();
+    QString fingerprint();
 
     void setEnabled( const bool enable );
     bool enabled() const { return m_bEnabled; }
 
 private:
     void updateHash();
-    QString fingerprint();
 
     bool dirty;
     bool hashdirty;
-    int m_Desk, m_Hash;
+    int m_Screen, m_Desk, m_Hash;
 
     QColor m_ColorA, defColorA;
     QColor m_ColorB, defColorB;
@@ -262,10 +275,12 @@ private:
     bool m_ReverseBlending, defReverseBlending;
     int m_MinOptimizationDepth;
     bool m_bShm;
+    bool m_bDrawBackgroundPerScreen;
 
     int m_MultiMode, defMultiMode;
     int m_Interval, m_LastChange;
     int m_CurrentWallpaper;
+    QString m_CurrentWallpaperName;
 
     KConfig *m_pConfig;
     KStandardDirs *m_pDirs;
@@ -298,11 +313,17 @@ public:
     int cacheSize() { return m_CacheSize; }
     void setCacheSize(int size);
 
+    bool drawBackgroundPerScreen(int desk) const;
+    void setDrawBackgroundPerScreen(int desk, bool perScreen);
+
     bool limitCache() { return m_bLimitCache; }
     void setLimitCache(bool limit);
 
-    bool commonBackground() { return m_bCommon; }
-    void setCommonBackground(bool common);
+    bool commonScreenBackground() { return m_bCommonScreen; }
+    void setCommonScreenBackground(bool common);
+
+    bool commonDeskBackground() { return m_bCommonDesk; }
+    void setCommonDeskBackground(bool common);
 
     bool dockPanel() { return m_bDock; }
     void setDockPanel(bool dock);
@@ -319,12 +340,20 @@ public:
     void setShadowEnabled(bool enabled);
     bool shadowEnabled() const { return m_shadowEnabled; }
 
+    void setTextLines(int lines);
+    int textLines() const { return m_textLines; }
+    void setTextWidth(int width);
+    int textWidth() const { return m_textWidth; }
+
     void readSettings();
     void writeSettings();
+    KConfig* getConfig() { return m_pConfig; }
 
 private:
     bool dirty;
-    bool m_bCommon, m_bDock;
+    bool m_bCommonDesk;
+    bool m_bCommonScreen;
+    bool m_bDock;
     bool m_bLimitCache, m_bExport;
     int m_CacheSize;
     QStringList m_Names;
@@ -332,9 +361,11 @@ private:
     QColor m_TextColor;
     QColor m_TextBackgroundColor;
     bool m_shadowEnabled;
-
+    int m_textLines;
+    int m_textWidth;
     KConfig *m_pConfig;
     bool m_bDeleteConfig;
+    QValueVector<bool> m_bDrawBackgroundPerScreen; // m_bDrawBackgroundPerScreen[desk]
 };
 
 
