@@ -1,6 +1,4 @@
 /* vi: ts=8 sts=4 sw=4
- *
- * $Id: icons.cpp,v 1.13 2001/07/20 17:57:50 antlarr Exp $
  *
  * This file is part of the KDE project, module kcmdisplay.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
@@ -12,38 +10,24 @@
  */
 
 #include <stdlib.h>
-#include <qwidget.h>
-#include <qlayout.h>
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qlabel.h>
-#include <qlistbox.h>
-#include <qlistview.h>                                                                     
-#include <qgroupbox.h>
-#include <qcombobox.h>
-#include <qpushbutton.h>
-#include <qslider.h>
-#include <qcheckbox.h>
-#include <qlabel.h>
-#include <qtabwidget.h>
-#include <qiconview.h>
 
-#include <kapp.h>
-#include <kdebug.h>
+#include <qcheckbox.h>
+#include <qcombobox.h>
+#include <qgroupbox.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qslider.h>
+
+#include <kcolorbutton.h>
 #include <kconfig.h>
-#include <kwin.h>
-#include <kglobal.h>
-#include <kstddirs.h>
+#include <kdebug.h>
 #include <kiconeffect.h>
-#include <kicontheme.h>
 #include <kiconloader.h>
-#include <klocale.h>
 #include <kipc.h>
-#include <kcolordlg.h>
-#include <kcolorbtn.h>
+#include <klocale.h>
 #include <kseparator.h>
 
-#include <icons.h>
+#include "icons.h"
 
 /**** KIconConfig ****/
 
@@ -70,7 +54,7 @@ KIconConfig::KIconConfig(QWidget *parent, const char *name)
 
     KSeparator *sep = new KSeparator( KSeparator::HLine, this );
     top->addWidget(sep, 1, 1);
-    // Preview at (2,0) - (2, 1) 
+    // Preview at (2,0) - (2, 1)
     QGridLayout *g_lay = new QGridLayout(4, 3, KDialog::marginHint(), 0);
     top->addMultiCellLayout(g_lay, 2, 2, 0, 1);
     g_lay->addRowSpacing(0, fontMetrics().lineSpacing());
@@ -103,12 +87,11 @@ KIconConfig::KIconConfig(QWidget *parent, const char *name)
 
     mpDPCheck = new QCheckBox(i18n("Double-sized pixels"), m_pTab1);
     connect(mpDPCheck, SIGNAL(toggled(bool)), SLOT(slotDPCheck(bool)));
-    grid->addWidget(mpDPCheck, 1, 0, Qt::AlignLeft);
+    grid->addMultiCellWidget(mpDPCheck, 1, 1, 0, 1, Qt::AlignLeft);
 
-    mpAlphaBCheck = new QCheckBox(i18n("Blend alpha channel"), m_pTab1);
-    connect(mpAlphaBCheck, SIGNAL(toggled(bool)), SLOT(slotAlphaBCheck(bool)));
-    grid->addWidget(mpAlphaBCheck, 2, 0, Qt::AlignLeft);
-    disableAlphaBlending = (QPixmap::defaultDepth()<=8);
+    mpAnimatedCheck = new QCheckBox(i18n("Animate icons"), m_pTab1);
+    connect(mpAnimatedCheck, SIGNAL(toggled(bool)), SLOT(slotAnimatedCheck(bool)));
+    grid->addMultiCellWidget(mpAnimatedCheck, 2, 2, 0, 1, Qt::AlignLeft);
 
     top->activate();
 
@@ -118,6 +101,11 @@ KIconConfig::KIconConfig(QWidget *parent, const char *name)
     preview();
 }
 
+KIconConfig::~KIconConfig()
+{
+  delete mpEffect;
+}
+
 QPushButton *KIconConfig::addPreviewIcon(int i, const QString &str, QWidget *parent, QGridLayout *lay)
 {
     QLabel *lab = new QLabel(str, parent);
@@ -126,7 +114,7 @@ QPushButton *KIconConfig::addPreviewIcon(int i, const QString &str, QWidget *par
     mpPreview[i]->setAlignment(AlignCenter);
     mpPreview[i]->setMinimumSize(105, 105);
     lay->addWidget(mpPreview[i], 2, i);
-    QPushButton *push = new QPushButton(i18n("Set Effect"), parent);
+    QPushButton *push = new QPushButton(i18n("Set Effect..."), parent);
     lay->addWidget(push, 3, i, AlignCenter);
     return push;
 }
@@ -142,11 +130,12 @@ void KIconConfig::init()
 	mbChanged[i] = false;
 
     // Fill list/checkboxen
-    mpUsageList->insertItem(i18n("Desktop / File Manager"));
+    mpUsageList->insertItem(i18n("Desktop/File Manager"));
     mpUsageList->insertItem(i18n("Toolbar"));
     mpUsageList->insertItem(i18n("Main Toolbar"));
     mpUsageList->insertItem(i18n("Small Icons"));
     mpUsageList->insertItem(i18n("Panel"));
+    mpUsageList->insertItem(i18n("All Icons"));
 
     // For reading the configuration
     mGroups += "Desktop";
@@ -164,7 +153,7 @@ void KIconConfig::initDefaults()
 {
     mDefaultEffect[0].type = KIconEffect::NoEffect;
     mDefaultEffect[1].type = KIconEffect::NoEffect;
-    mDefaultEffect[2].type = KIconEffect::NoEffect;
+    mDefaultEffect[2].type = KIconEffect::ToGray;
     mDefaultEffect[0].transparant = false;
     mDefaultEffect[1].transparant = false;
     mDefaultEffect[2].transparant = true;
@@ -174,20 +163,32 @@ void KIconConfig::initDefaults()
     mDefaultEffect[0].color = QColor(144,128,248);
     mDefaultEffect[1].color = QColor(169,156,255);
     mDefaultEffect[2].color = QColor(34,202,0);
+    mDefaultEffect[0].color2 = QColor(0,0,0);
+    mDefaultEffect[1].color2 = QColor(0,0,0);
+    mDefaultEffect[2].color2 = QColor(0,0,0);
 
-    int i;
+    const int defDefSizes[] = { 32, 22, 22, 16, 32 };
+
+    KIcon::Group i;
     QStringList::ConstIterator it;
-    for(it=mGroups.begin(), i=0; it!=mGroups.end(); it++, i++)
+    for(it=mGroups.begin(), i=KIcon::FirstGroup; it!=mGroups.end(); ++it, i++)
     {
 	mbDP[i] = false;
-	mbAlphaB[i] = false;
 	mbChanged[i] = true;
-	mSizes[i] = mpTheme->defaultSize(i);
+	mbAnimated[i] = false;
+	if (mpTheme)
+	    mSizes[i] = mpTheme->defaultSize(i);
+	else
+	    mSizes[i] = defDefSizes[i];
 
 	mEffects[i][0] = mDefaultEffect[0];
 	mEffects[i][1] = mDefaultEffect[1];
 	mEffects[i][2] = mDefaultEffect[2];
     }
+    // Animate desktop icons by default
+    int group = mGroups.findIndex( "Desktop" );
+    if ( group != -1 )
+        mbAnimated[group] = true;
 
     // This is the new default in KDE 2.2, in sync with the kiconeffect of kdelibs Nolden 2001/06/11
     int activeState = mStates.findIndex( "Active" );
@@ -211,26 +212,37 @@ void KIconConfig::initDefaults()
 
 void KIconConfig::read()
 {
-    for (int i=0; i<KIcon::LastGroup; i++)
-	mAvSizes[i] = mpTheme->querySizes(i);
+    if (mpTheme)
+    {
+        for (KIcon::Group i=KIcon::FirstGroup; i<KIcon::LastGroup; i++)
+            mAvSizes[i] = mpTheme->querySizes(i);
 
-    mTheme = mpTheme->current();
-    mExample = mpTheme->example();
+        mTheme = mpTheme->current();
+        mExample = mpTheme->example();
+    }
+    else
+    {
+        for (KIcon::Group i=KIcon::FirstGroup; i<KIcon::LastGroup; i++)
+            mAvSizes[i] = QValueList<int>();
+
+        mTheme = QString::null;
+        mExample = QString::null;
+    }
 
     initDefaults();
 
     int i, j, effect;
     QStringList::ConstIterator it, it2;
-    for (it=mGroups.begin(), i=0; it!=mGroups.end(); it++, i++)
+    for (it=mGroups.begin(), i=0; it!=mGroups.end(); ++it, i++)
     {
         mbChanged[i] = false;
 
 	mpConfig->setGroup(*it + "Icons");
 	mSizes[i] = mpConfig->readNumEntry("Size", mSizes[i]);
 	mbDP[i] = mpConfig->readBoolEntry("DoublePixels", mbDP[i]);
-	mbAlphaB[i] = mpConfig->readBoolEntry("AlphaBlending", mbAlphaB[i]);
+	mbAnimated[i] = mpConfig->readBoolEntry("Animated", mbAnimated[i]);
 
-	for (it2=mStates.begin(), j=0; it2!=mStates.end(); it2++, j++)
+	for (it2=mStates.begin(), j=0; it2!=mStates.end(); ++it2, j++)
 	{
 	    QString tmp = mpConfig->readEntry(*it2 + "Effect");
 	    if (tmp == "togray")
@@ -241,12 +253,15 @@ void KIconConfig::read()
 		effect = KIconEffect::ToGamma;
 	    else if (tmp == "desaturate")
 		effect = KIconEffect::DeSaturate;
+	    else if (tmp == "tomonochrome")
+		effect = KIconEffect::ToMonochrome;
 	    else if (tmp == "none")
 		effect = KIconEffect::NoEffect;
 	    else continue;
 	    mEffects[i][j].type = effect;
 	    mEffects[i][j].value = mpConfig->readDoubleNumEntry(*it2 + "Value");
 	    mEffects[i][j].color = mpConfig->readColorEntry(*it2 + "Color");
+	    mEffects[i][j].color2 = mpConfig->readColorEntry(*it2 + "Color2");
 	    mEffects[i][j].transparant = mpConfig->readBoolEntry(*it2 + "SemiTransparent");
 	}
     }
@@ -256,46 +271,51 @@ void KIconConfig::apply()
 {
     mpUsageList->setCurrentItem(mUsage);
 
-    int delta = 1000, dw, index = -1, size, i;
+    int delta = 1000, dw, index = -1, size = 0, i;
     QValueList<int>::Iterator it;
     mpSizeBox->clear();
-    for (it=mAvSizes[mUsage].begin(), i=0; it!=mAvSizes[mUsage].end(); it++, i++)
-    {
-	mpSizeBox->insertItem(QString().setNum(*it));
-	dw = abs(mSizes[mUsage] - *it);
-	if (dw < delta)
-	{
-	    delta = dw;
-	    index = i;
-	    size = *it;
-	}
+    if (mUsage < KIcon::LastGroup) {
+        for (it=mAvSizes[mUsage].begin(), i=0; it!=mAvSizes[mUsage].end(); ++it, i++)
+        {
+            mpSizeBox->insertItem(QString().setNum(*it));
+            dw = abs(mSizes[mUsage] - *it);
+            if (dw < delta)
+            {
+                delta = dw;
+                index = i;
+                size = *it;
+            }
 
+        }
+        if (index != -1)
+        {
+            mpSizeBox->setCurrentItem(index);
+            mSizes[mUsage] = size; // best or exact match
+        }
+        mpDPCheck->setChecked(mbDP[mUsage]);
+        mpAnimatedCheck->setChecked(mbAnimated[mUsage]);
     }
-    if (index != -1)
-    {
-	mpSizeBox->setCurrentItem(index);
-	mSizes[mUsage] = size; // best or exact match
-    }
-    mpDPCheck->setChecked(mbDP[mUsage]);
-    mpAlphaBCheck->setChecked(mbAlphaB[mUsage]);
 }
 
 void KIconConfig::preview(int i)
 {
     // Apply effects ourselves because we don't want to sync
     // the configuration every preview.
-    QPixmap pm = mpLoader->loadIcon(mExample, KIcon::NoGroup, mSizes[mUsage]);
+
+    int viewedGroup = (mUsage == KIcon::LastGroup) ? KIcon::FirstGroup : mUsage;
+
+    QPixmap pm = mpLoader->loadIcon(mExample, KIcon::NoGroup, mSizes[viewedGroup]);
     QImage img = pm.convertToImage();
-    if (mbDP[mUsage])
+    if (mbDP[viewedGroup])
     {
 	int w = img.width() * 2;
 	img = img.smoothScale(w, w);
     }
 
-    Effect &effect = mEffects[mUsage][i]; 
+    Effect &effect = mEffects[viewedGroup][i];
 
     img = mpEffect->apply(img, effect.type,
-	    effect.value, effect.color, effect.transparant);
+	    effect.value, effect.color, effect.color2, effect.transparant);
     pm.convertFromImage(img);
     mpPreview[i]->setPixmap(pm);
 }
@@ -309,11 +329,19 @@ void KIconConfig::preview()
 
 void KIconConfig::load()
 {
+    load( false );
+}
+
+void KIconConfig::load( bool useDefaults )
+{
+    mpConfig = KGlobal::config();
+    mpConfig->setReadDefaults( useDefaults );
     read();
     apply();
-    emit changed(false);
     for (int i=0; i<KIcon::LastGroup; i++)
 	mbChanged[i] = false;
+    preview();
+    emit changed( useDefaults );
 }
 
 
@@ -321,13 +349,13 @@ void KIconConfig::save()
 {
     int i, j;
     QStringList::ConstIterator it, it2;
-    for (it=mGroups.begin(), i=0; it!=mGroups.end(); it++, i++)
+    for (it=mGroups.begin(), i=0; it!=mGroups.end(); ++it, i++)
     {
 	mpConfig->setGroup(*it + "Icons");
 	mpConfig->writeEntry("Size", mSizes[i], true, true);
 	mpConfig->writeEntry("DoublePixels", mbDP[i], true, true);
-	mpConfig->writeEntry("AlphaBlending", mbAlphaB[i], true, true);
-	for (it2=mStates.begin(), j=0; it2!=mStates.end(); it2++, j++)
+	mpConfig->writeEntry("Animated", mbAnimated[i], true, true);
+	for (it2=mStates.begin(), j=0; it2!=mStates.end(); ++it2, j++)
 	{
 	    QString tmp;
 	    switch (mEffects[i][j].type)
@@ -344,6 +372,9 @@ void KIconConfig::save()
 	    case KIconEffect::DeSaturate:
 		tmp = "desaturate";
 		break;
+	    case KIconEffect::ToMonochrome:
+		tmp = "tomonochrome";
+		break;
 	    default:
 		tmp = "none";
 		break;
@@ -351,6 +382,7 @@ void KIconConfig::save()
 	    mpConfig->writeEntry(*it2 + "Effect", tmp, true, true);
 	    mpConfig->writeEntry(*it2 + "Value", mEffects[i][j].value, true, true);
             mpConfig->writeEntry(*it2 + "Color", mEffects[i][j].color, true, true);
+            mpConfig->writeEntry(*it2 + "Color2", mEffects[i][j].color2, true, true);
             mpConfig->writeEntry(*it2 + "SemiTransparent", mEffects[i][j].transparant, true, true);
 	}
     }
@@ -372,31 +404,24 @@ void KIconConfig::save()
 
 void KIconConfig::defaults()
 {
-    initDefaults();
-    apply();
-    preview();
-    emit changed(true);
+    load( true );
 }
 
 void KIconConfig::slotUsage(int index)
 {
     mUsage = index;
-    if ( mUsage == KIcon::Panel )
+    if ( mUsage == KIcon::Panel || mUsage == KIcon::LastGroup )
     {
         mpSizeBox->setEnabled(false);
         mpDPCheck->setEnabled(false);
-	if (!disableAlphaBlending)
-	    mpAlphaBCheck->setEnabled(true);
-    } 
+	mpAnimatedCheck->setEnabled( mUsage == KIcon::Panel );
+    }
     else
     {
         mpSizeBox->setEnabled(true);
         mpDPCheck->setEnabled(true);
-	if (!disableAlphaBlending)
-	    mpAlphaBCheck->setEnabled( (mUsage==KIcon::Desktop)? true : false);
+	mpAnimatedCheck->setEnabled( mUsage == KIcon::Desktop );
     }
-    if (disableAlphaBlending)
-	mpAlphaBCheck->setEnabled(false);
 
     apply();
     preview();
@@ -404,9 +429,11 @@ void KIconConfig::slotUsage(int index)
 
 void KIconConfig::EffectSetup(int state)
 {
-    QPixmap pm = mpLoader->loadIcon(mExample, KIcon::NoGroup, mSizes[mUsage]);
+    int viewedGroup = (mUsage == KIcon::LastGroup) ? KIcon::FirstGroup : mUsage;
+
+    QPixmap pm = mpLoader->loadIcon(mExample, KIcon::NoGroup, mSizes[viewedGroup]);
     QImage img = pm.convertToImage();
-    if (mbDP[mUsage])
+    if (mbDP[viewedGroup])
     {
 	int w = img.width() * 2;
 	img = img.smoothScale(w, w);
@@ -419,20 +446,36 @@ void KIconConfig::EffectSetup(int state)
     case 1 : caption = i18n("Setup Active Icon Effect"); break;
     case 2 : caption = i18n("Setup Disabled Icon Effect"); break;
     }
-   
-    KIconEffectSetupDialog dlg(mEffects[mUsage][state], mDefaultEffect[state], caption, img);
+
+    KIconEffectSetupDialog dlg(mEffects[viewedGroup][state], mDefaultEffect[state], caption, img);
 
     if (dlg.exec() == QDialog::Accepted)
     {
-        mEffects[mUsage][state] = dlg.effect();
+        if (mUsage == KIcon::LastGroup) {
+            for (int i=0; i<KIcon::LastGroup; i++)
+                mEffects[i][state] = dlg.effect();
+        } else {
+            mEffects[mUsage][state] = dlg.effect();
+        }
+
+        // AK - can this call be moved therefore removing
+        //      code duplication?
+
         emit changed(true);
-        mbChanged[mUsage] = true;
+
+        if (mUsage == KIcon::LastGroup) {
+            for (int i=0; i<KIcon::LastGroup; i++)
+                mbChanged[i] = true;
+        } else {
+            mbChanged[mUsage] = true;
+        }
     }
     preview(state);
 }
 
 void KIconConfig::slotSize(int index)
 {
+    Q_ASSERT(mUsage < KIcon::LastGroup);
     mSizes[mUsage] = mAvSizes[mUsage][index];
     preview();
     emit changed(true);
@@ -441,31 +484,36 @@ void KIconConfig::slotSize(int index)
 
 void KIconConfig::slotDPCheck(bool check)
 {
-    mbDP[mUsage] = check;
+    Q_ASSERT(mUsage < KIcon::LastGroup);
+    if (mbDP[mUsage] != check)
+    {
+        mbDP[mUsage] = check;
+        emit changed(true);
+        mbChanged[mUsage] = true;
+    }
     preview();
-    emit changed(true);
-    mbChanged[mUsage] = true;
+
 }
 
-void KIconConfig::slotAlphaBCheck(bool check)
+void KIconConfig::slotAnimatedCheck(bool check)
 {
-    mbAlphaB[mUsage] = check;
- // XXX Should warning the user that this change will only
- // take effect when restarting KDE.
-
- //    preview();
-    emit changed(true);
-    mbChanged[mUsage] = true;
+    Q_ASSERT(mUsage < KIcon::LastGroup);
+    if (mbAnimated[mUsage] != check)
+    {
+        mbAnimated[mUsage] = check;
+        emit changed(true);
+        mbChanged[mUsage] = true;
+    }
 }
 
-KIconEffectSetupDialog::KIconEffectSetupDialog(const Effect &effect, 
-    const Effect &defaultEffect, 
-    const QString &caption, const QImage &image, 
+KIconEffectSetupDialog::KIconEffectSetupDialog(const Effect &effect,
+    const Effect &defaultEffect,
+    const QString &caption, const QImage &image,
     QWidget *parent, char *name)
     : KDialogBase(parent, name, true, caption,
-	Default|Ok|Cancel, Ok, true), 
-      mEffect(effect), 
-      mDefaultEffect(defaultEffect), 
+	Default|Ok|Cancel, Ok, true),
+      mEffect(effect),
+      mDefaultEffect(defaultEffect),
       mExample(image)
 {
     mpEffect = new KIconEffect;
@@ -477,7 +525,7 @@ KIconEffectSetupDialog::KIconEffectSetupDialog(const Effect &effect,
     QWidget *page = new QWidget(this);
     setMainWidget(page);
 
-    QGridLayout *top = new QGridLayout(page, 4, 3, 0, spacingHint());
+    QGridLayout *top = new QGridLayout(page, 4, 2, 0, spacingHint());
     top->setColStretch(0,1);
     top->addColSpacing(1,10);
     top->setColStretch(2,2);
@@ -492,16 +540,18 @@ KIconEffectSetupDialog::KIconEffectSetupDialog(const Effect &effect,
     mpEffectBox->insertItem(i18n("Colorize"));
     mpEffectBox->insertItem(i18n("Gamma"));
     mpEffectBox->insertItem(i18n("Desaturate"));
+    mpEffectBox->insertItem(i18n("To Monochrome"));
+    mpEffectBox->setMinimumWidth( 100 );
     connect(mpEffectBox, SIGNAL(highlighted(int)), SLOT(slotEffectType(int)));
     top->addMultiCellWidget(mpEffectBox, 1, 2, 0, 0, Qt::AlignLeft);
     lbl->setBuddy(mpEffectBox);
 
-    mpSTCheck = new QCheckBox(i18n("Semi-transparent"), page);
+    mpSTCheck = new QCheckBox(i18n("&Semi-transparent"), page);
     connect(mpSTCheck, SIGNAL(toggled(bool)), SLOT(slotSTCheck(bool)));
     top->addWidget(mpSTCheck, 3, 0, Qt::AlignLeft);
-    
+
     frame = new QGroupBox(i18n("Preview"), page);
-    top->addMultiCellWidget(frame, 0, 1, 2, 2);
+    top->addMultiCellWidget(frame, 0, 1, 1, 1);
     grid = new QGridLayout(frame, 2, 1, marginHint(), spacingHint());
     grid->addRowSpacing(0, fontMetrics().lineSpacing());
     grid->setRowStretch(1, 1);
@@ -511,35 +561,52 @@ KIconEffectSetupDialog::KIconEffectSetupDialog(const Effect &effect,
     mpPreview->setMinimumSize(105, 105);
     grid->addWidget(mpPreview, 1, 0);
 
-    frame = new QGroupBox(i18n("Effect Parameters"), page);
-    top->addMultiCellWidget(frame, 2, 3, 2, 2);
-    grid = new QGridLayout(frame, 3, 2, marginHint(), spacingHint());
+    mpEffectGroup = new QGroupBox(i18n("Effect Parameters"), page);
+    top->addMultiCellWidget(mpEffectGroup, 2, 3, 1, 1);
+    grid = new QGridLayout(mpEffectGroup, 3, 2, marginHint(), spacingHint());
     grid->addRowSpacing(0, fontMetrics().lineSpacing());
 
-    lbl = new QLabel(i18n("Amount:"), frame);
-    grid->addWidget(lbl, 1, 0);
-    mpEffectSlider = new QSlider(0, 100, 5, 10, QSlider::Horizontal, frame);
+    mpEffectLabel = new QLabel(i18n("&Amount:"), mpEffectGroup);
+    grid->addWidget(mpEffectLabel, 1, 0);
+    mpEffectSlider = new QSlider(0, 100, 5, 10, QSlider::Horizontal, mpEffectGroup);
+    mpEffectLabel->setBuddy( mpEffectSlider );
     connect(mpEffectSlider, SIGNAL(valueChanged(int)), SLOT(slotEffectValue(int)));
     grid->addWidget(mpEffectSlider, 1, 1);
 
-    lbl = new QLabel(i18n("Color:"), frame);
-    grid->addWidget(lbl, 2, 0);
-    mpEColButton = new KColorButton(frame);
+    mpEffectColor = new QLabel(i18n("Co&lor:"), mpEffectGroup);
+    grid->addWidget(mpEffectColor, 2, 0);
+    mpEColButton = new KColorButton(mpEffectGroup);
+    mpEffectColor->setBuddy( mpEColButton );
     connect(mpEColButton, SIGNAL(changed(const QColor &)),
 		SLOT(slotEffectColor(const QColor &)));
     grid->addWidget(mpEColButton, 2, 1);
 
-    init(); 
+    mpEffectColor2 = new QLabel(i18n("&Second color:"), mpEffectGroup);
+    grid->addWidget(mpEffectColor2, 3, 0);
+    mpECol2Button = new KColorButton(mpEffectGroup);
+    mpEffectColor2->setBuddy( mpECol2Button );
+    connect(mpECol2Button, SIGNAL(changed(const QColor &)),
+		SLOT(slotEffectColor2(const QColor &)));
+    grid->addWidget(mpECol2Button, 3, 1);
+
+    init();
     preview();
 }
 
+KIconEffectSetupDialog::~KIconEffectSetupDialog()
+{
+  delete mpEffect;
+}
+
 void KIconEffectSetupDialog::init()
 {
     mpEffectBox->setCurrentItem(mEffect.type);
     mpEffectSlider->setEnabled(mEffect.type != KIconEffect::NoEffect);
-    mpEColButton->setEnabled(mEffect.type == KIconEffect::Colorize);
+    mpEColButton->setEnabled(mEffect.type == KIconEffect::Colorize || mEffect.type == KIconEffect::ToMonochrome);
+    mpECol2Button->setEnabled(mEffect.type == KIconEffect::ToMonochrome);
     mpEffectSlider->setValue((int) (100.0 * mEffect.value + 0.5));
     mpEColButton->setColor(mEffect.color);
+    mpECol2Button->setColor(mEffect.color2);
     mpSTCheck->setChecked(mEffect.transparant);
 }
 
@@ -555,11 +622,21 @@ void KIconEffectSetupDialog::slotEffectColor(const QColor &col)
      preview();
 }
 
+void KIconEffectSetupDialog::slotEffectColor2(const QColor &col)
+{
+     mEffect.color2 = col;
+     preview();
+}
+
 void KIconEffectSetupDialog::slotEffectType(int type)
 {
     mEffect.type = type;
+    mpEffectGroup->setEnabled(mEffect.type != KIconEffect::NoEffect);
     mpEffectSlider->setEnabled(mEffect.type != KIconEffect::NoEffect);
-    mpEColButton->setEnabled(mEffect.type == KIconEffect::Colorize);
+    mpEffectColor->setEnabled(mEffect.type == KIconEffect::Colorize || mEffect.type == KIconEffect::ToMonochrome);
+    mpEColButton->setEnabled(mEffect.type == KIconEffect::Colorize || mEffect.type == KIconEffect::ToMonochrome);
+    mpEffectColor2->setEnabled(mEffect.type == KIconEffect::ToMonochrome);
+    mpECol2Button->setEnabled(mEffect.type == KIconEffect::ToMonochrome);
     preview();
 }
 
@@ -581,7 +658,7 @@ void KIconEffectSetupDialog::preview()
     QPixmap pm;
     QImage img = mExample.copy();
     img = mpEffect->apply(img, mEffect.type,
-          mEffect.value, mEffect.color, mEffect.transparant);
+          mEffect.value, mEffect.color, mEffect.color2, mEffect.transparant);
     pm.convertFromImage(img);
     mpPreview->setPixmap(pm);
 }
