@@ -12,38 +12,39 @@
 #ifndef __icons_h__
 #define __icons_h__
 
-#include <qvaluelist.h>
 #include <qcolor.h>
 #include <qimage.h>
+#include <qvaluelist.h>
 
 #include <kcmodule.h>
 #include <kdialogbase.h>
 
-class QColor;
-class QWidget;
 class QCheckBox;
+class QColor;
 class QComboBox;
+class QGridLayout;
+class QGroupBox;
+class QIconView;
+class QLabel;
 class QListBox;
 class QListView;
-class QSlider;
-class QLabel;
-class QIconView;
-class QTabWidget;
-class QGridLayout;
 class QPushButton;
-class QGroupBox;
+class QSlider;
+class QTabWidget;
+class QWidget;
 
+class KColorButton;
 class KConfig;
 class KIconEffect;
-class KIconTheme;
 class KIconLoader;
-class KColorButton;
+class KIconTheme;
 
 struct Effect 
 {
     int type;
     float value;
     QColor color;
+    QColor color2;
     bool transparant;
 };
 
@@ -60,6 +61,7 @@ public:
     ~KIconConfig();
 
     virtual void load();
+    virtual void load( bool useDefaults );
     virtual void save();
     virtual void defaults();
     void preview();
@@ -128,6 +130,7 @@ protected:
 protected slots:
     void slotEffectValue(int value);
     void slotEffectColor(const QColor &col);
+    void slotEffectColor2(const QColor &col);
     void slotEffectType(int type);
     void slotSTCheck(bool b);
     void slotDefault();
@@ -138,11 +141,12 @@ private:
     QCheckBox *mpSTCheck;
     QSlider *mpEffectSlider;
     KColorButton *mpEColButton;
+    KColorButton *mpECol2Button;
     Effect mEffect;
     Effect mDefaultEffect;
     QImage mExample;
     QGroupBox *mpEffectGroup;
-    QLabel *mpPreview, *mpEffectLabel, *mpEffectColor;
+    QLabel *mpPreview, *mpEffectLabel, *mpEffectColor, *mpEffectColor2;
 };                      
                       
 #endif
