@@ -149,7 +149,7 @@ void KCookiesManagement::save()
     {
       QString caption = i18n ("DCOP Communication Error");
       QString message = i18n ("Unable to delete all the cookies as requested.");
-      KMessageBox::sorry (this, caption, message);
+      KMessageBox::sorry (this, message,caption);
       return;
     }
     m_bDeleteAll = false; // deleted[Cookies|Domains] have been cleared yet    
@@ -169,7 +169,7 @@ void KCookiesManagement::save()
     {
       QString caption = i18n ("DCOP Communication Error");
       QString message = i18n ("Unable to delete cookies as requested.");
-      KMessageBox::sorry (this, caption, message);
+      KMessageBox::sorry (this, message,caption);
       return;
     }
 
@@ -250,7 +250,7 @@ void KCookiesManagement::getDomains()
     QString caption = i18n ("Information Lookup Failure");
     QString message = i18n ("Unable to retrieve information about the "
                             "cookies stored on your computer.");
-    KMessageBox::sorry (this, caption, message);
+    KMessageBox::sorry (this, message, caption);
     return;
   }
 
