@@ -652,7 +652,7 @@ void KBackgroundManager::setCommon(int common)
 // DCOP exported
 void KBackgroundManager::setWallpaper(QString wallpaper, int mode)
 {
-    if (mode < 0 || mode >= KBackgroundSettings::lastBackgroundMode) {
+    if (mode < 0 || mode >= KBackgroundSettings::lastWallpaperMode) {
       kdDebug() << "Invalid background mode " << mode << " passed to " << k_funcinfo << "\n";
       return;
     }
@@ -714,7 +714,7 @@ void KBackgroundManager::setCache( int bLimit, int size )
 // DCOP exported
 void KBackgroundManager::setWallpaper(int desk, QString wallpaper, int mode)
 {
-    if (mode < 0 || mode >= KBackgroundSettings::lastBackgroundMode) {
+    if (mode < 0 || mode >= KBackgroundSettings::lastWallpaperMode) {
       kdDebug() << "Invalid background mode " << mode << " passed to " << k_funcinfo << "\n";
       return;
     }
