@@ -128,16 +128,11 @@ QPixmap QuickURL::pixmap( mode_t _mode, KIcon::Group _group,
 
 QuickButton::QuickButton(const QString &u, KAction* configAction, 
                          QWidget *parent, const char *name) : 
-     QButton(parent, name),
+     SimpleButton(parent, name),
      m_flashCounter(0),
      m_sticky(false)
 {
     installEventFilter(KickerTip::the());
-    if (parent && ! parent->parent())
-    {
-        setBackgroundMode(X11ParentRelative);
-    }
-    setBackgroundOrigin( AncestorOrigin );
     setMouseTracking(true);
     _highlight = false;
     _oldCursor = cursor();
@@ -163,14 +158,9 @@ QuickButton::QuickButton(const QString &u, KAction* configAction,
     m_stickyAction->plug(_popup, 2);
     m_stickyId = _popup->idAt(2);
 
-    settingsChanged(KApplication::SETTINGS_MOUSE);
-    connect(kapp, SIGNAL(settingsChanged(int)), SLOT(settingsChanged(int)));
-    connect(kapp, SIGNAL(iconChanged(int)), SLOT(iconChanged(int)));
     connect(this, SIGNAL(clicked()), SLOT(launch()));
     connect(this, SIGNAL(removeApp(QuickButton *)), parent,
         SLOT(removeAppManually(QuickButton *)));
-    kapp->addKipcEventMask(KIPC::SettingsChanged);
-    kapp->addKipcEventMask(KIPC::IconChanged);
 }
 
 QuickButton::~QuickButton()
@@ -188,59 +178,21 @@ QString QuickButton::url() const
 QString QuickButton::menuId() const
 {  return _qurl->menuId();}
 
+
 void QuickButton::loadIcon()
-{  // Set Icon Dimension from size
+{ 
+    // Set Icon Dimension from size
    _iconDim=std::min(size().width(),size().height())-2*ICON_MARGIN;
    // Load icons
    _icon = _qurl->pixmap(0, KIcon::Panel, _iconDim, KIcon::DefaultState);
    _iconh = _qurl->pixmap(0, KIcon::Panel, _iconDim, KIcon::ActiveState);
+   setPixmap(_icon);
 }
 
-void QuickButton::resizeEvent(QResizeEvent *)
+void QuickButton::resizeEvent(QResizeEvent *e)
 {
    loadIcon();
-}
-
-void QuickButton::paintEvent(QPaintEvent *)
-{
-   QPainter p(this);
-   drawButton(&p);
-}
-
-void QuickButton::drawButton(QPainter *p)
-{
-   if (isDown() || isOn()) {
-      p->fillRect(rect(), colorGroup().brush(QColorGroup::Mid));
-      qDrawWinButton(p, 0, 0, width(), height(), colorGroup(), true);
-   }
-
-   drawButtonLabel(p);
-}
-
-void QuickButton::drawButtonLabel(QPainter *p)
-{
-   QPixmap *pix = &_icon;    //PIX
-   if (_highlight) pix = &_iconh;
-   QPoint offset=QPoint((width()-_iconDim)/2,(height()-_iconDim)/2);
-   int d = ICON_MARGIN;
-   if (isDown() || isOn()) d += 1;
-
-   if (m_flashCounter%500<250) p->drawPixmap(offset.x()+ d, offset.y() + d, *pix);
-}
-
-void QuickButton::enterEvent(QEvent*)
-{
-   if (_changeCursorOverItem)
-      setCursor(KCursor().handCursor());
-   _highlight = true;
-   repaint();
-}
-
-void QuickButton::leaveEvent(QEvent*)
-{
-   if (_changeCursorOverItem)
-      setCursor(_oldCursor);
-   setDragging(false);
+   SimpleButton::resizeEvent(e);
 }
 
 void QuickButton::mousePressEvent(QMouseEvent *e)
@@ -276,26 +228,17 @@ void QuickButton::mouseMoveEvent(QMouseEvent *e)
    }
 }
 
-void QuickButton::settingsChanged(int category)
-{
-   if (category != KApplication::SETTINGS_MOUSE) return;
-
-   _changeCursorOverItem = KGlobalSettings::changeCursorOverIcon();
-
-   if (!_changeCursorOverItem)
-      setCursor(_oldCursor);
-}
-
-void QuickButton::iconChanged(int)
+void QuickButton::slotIconChanged(int group)
 {
    loadIcon();
-   repaint();
+   SimpleButton::slotIconChanged(group);
+   update();
 }
 
 void QuickButton::launch()
 {
    setDown(false);
-   repaint();
+   update();
    KIconEffect::visualActivate(this, rect());
    _qurl->run();
    emit executed(_qurl->menuId());
@@ -305,7 +248,7 @@ void QuickButton::setDragging(bool enable)
 {
    setDown(enable);
    _highlight=enable;
-   repaint();
+   update();
 }
 
 void QuickButton::setEnableDrag(bool enable)
@@ -331,7 +274,7 @@ void QuickButton::slotFlash()
     {
         m_flashCounter -= timeout;
         if (m_flashCounter < 0) m_flashCounter = 0;
-        repaint();
+        update();
         QTimer::singleShot(timeout, this, SLOT(slotFlash()));
     }
 }
