@@ -89,6 +89,7 @@ KMiniPager::KMiniPager(const QString& configFile, Type type, int actions,
       m_contextMenu(0),
       m_settings( new PagerSettings(sharedConfig()) )
 {
+    setBackgroundOrigin( AncestorOrigin );
     int scnum = QApplication::desktop()->screenNumber(this);
     QRect desk = QApplication::desktop()->screenGeometry(scnum);
     if (desk.width() <= 800)
@@ -107,6 +108,7 @@ KMiniPager::KMiniPager(const QString& configFile, Type type, int actions,
     }
 
     m_group = new QButtonGroup(this);
+    m_group->setBackgroundOrigin(AncestorOrigin);
     m_group->setFrameStyle(QFrame::NoFrame);
     m_group->setExclusive( true );
 
@@ -165,6 +167,7 @@ KMiniPager::~KMiniPager()
     KGlobal::locale()->removeCatalogue("kminipagerapplet");
     delete m_contextMenu;
     delete m_settings;
+    delete m_shadowEngine;
 }
 
 void KMiniPager::slotBackgroundChanged(int desk)
@@ -652,17 +655,10 @@ KWin::WindowInfo* KMiniPager::info( WId win )
     return m_windows[win];
 }
 
-KShadowEngine* KMiniPager::shadowEngine()
+KTextShadowEngine* KMiniPager::shadowEngine()
 {
     if (!m_shadowEngine)
-    {
-        KShadowSettings * shadset = new KShadowSettings();
-        shadset->setOffsetX(0);
-        shadset->setOffsetY(0);
-        shadset->setThickness(1);
-        shadset->setMaxOpacity(96);
-        m_shadowEngine = new KShadowEngine(shadset);
-    }
+        m_shadowEngine = new KTextShadowEngine();
 
     return m_shadowEngine;
 }
@@ -907,3 +903,4 @@ void KMiniPager::applicationRegistered( const QCString  & appName )
         showKPager(false);
     }
 }
+
