@@ -123,6 +123,7 @@ Kicker::Kicker()
 
     connect(this, SIGNAL(settingsChanged(int)), SLOT(slotSettingsChanged(int)));
     connect(this, SIGNAL(kdisplayPaletteChanged()), SLOT(paletteChanged()));
+    connect(this, SIGNAL(kdisplayStyleChanged()), SLOT(slotStyleChanged()));
 
 #if (QT_VERSION-0 >= 0x030200) // XRANDR support
     connect(desktop(), SIGNAL(resized(int)), SLOT(slotDesktopResized()));
@@ -199,6 +200,11 @@ void Kicker::paletteChanged()
     KickerSettings::self()->writeConfig();
 }
 
+void Kicker::slotStyleChanged()
+{
+	restart();
+}
+
 bool Kicker::highlightMenuItem(const QString &menuId)
 {
     return MenuManager::the()->kmenu()->highlightMenuItem( menuId );
