@@ -1,6 +1,4 @@
 /*
-	$Id: kwindecoration.cpp,v 1.52 2004/01/11 14:30:10 zander Exp $
-
 	This is the new kwindecoration kcontrol module
 
 	Copyright (c) 2001
@@ -17,15 +15,15 @@
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-  
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-  
+
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */
 
@@ -73,52 +71,54 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const char* name, co
 	kwinConfig.setGroup("Style");
         plugins = new KDecorationPreviewPlugins( &kwinConfig );
 
-	QVBoxLayout* layout = new QVBoxLayout(this);
+	QVBoxLayout* layout = new QVBoxLayout(this, 0, KDialog::spacingHint()); 
+
+// Save this for later...
+//	cbUseMiniWindows = new QCheckBox( i18n( "Render mini &titlebars for all windows"), checkGroup );
+//	QWhatsThis::add( cbUseMiniWindows, i18n( "Note that this option is not available on all styles yet!" ) );
+
 	tabWidget = new QTabWidget( this );
 	layout->addWidget( tabWidget );
 
 	// Page 1 (General Options)
 	QWidget *pluginPage = new QWidget( tabWidget );
 
-	QHBox *hbox = new QHBox(pluginPage);
-	hbox->setSpacing(KDialog::spacingHint());
-//	QLabel *lbl = new QLabel( i18n("&Decoration:"), hbox );
-	decorationList = new KComboBox( hbox );
-//	lbl->setBuddy(decorationList);
+	QVBoxLayout* pluginLayout = new QVBoxLayout(pluginPage, KDialog::marginHint(), KDialog::spacingHint());
+
+	// decoration chooser
+	decorationList = new KComboBox( pluginPage );
 	QString whatsThis = i18n("Select the window decoration. This is the look and feel of both "
                              "the window borders and the window handle.");
-//	QWhatsThis::add(lbl, whatsThis);
 	QWhatsThis::add(decorationList, whatsThis);
+	pluginLayout->addWidget(decorationList);
 
-	QVBoxLayout* pluginLayout = new QVBoxLayout(pluginPage, KDialog::marginHint(), KDialog::spacingHint());
-	pluginLayout->addWidget(hbox);
-
-// Save this for later...
-//	cbUseMiniWindows = new QCheckBox( i18n( "Render mini &titlebars for all windows"), checkGroup );
-//	QWhatsThis::add( cbUseMiniWindows, i18n( "Note that this option is not available on all styles yet!" ) );
-
-	QFrame* preview_frame = new QFrame( pluginPage );
-	preview_frame->setFrameShape( QFrame::NoFrame );
-	QVBoxLayout* preview_layout = new QVBoxLayout( preview_frame, 0 );
-	preview = new KDecorationPreview( preview_frame );
-	preview_layout->addWidget( preview );
-	pluginLayout->addWidget( preview_frame );
-	pluginLayout->setStretchFactor( preview_frame, 10 );
-
-	pluginSettingsGrp = new QGroupBox( i18n("Decoration Options"), pluginPage );
+	QGroupBox *pluginSettingsGrp = new QGroupBox( i18n("Decoration Options"), pluginPage );
 	pluginSettingsGrp->setColumnLayout( 0, Vertical );
 	pluginSettingsGrp->setFlat( true );
 	pluginSettingsGrp->layout()->setMargin( 0 );
 	pluginSettingsGrp->layout()->setSpacing( KDialog::spacingHint() );
 	pluginLayout->addWidget( pluginSettingsGrp );
 
+	pluginLayout->addStretch();
+
+	// Border size chooser
+	lBorder = new QLabel (i18n("B&order size:"), pluginSettingsGrp);
+	cBorder = new QComboBox(pluginSettingsGrp);
+	lBorder->setBuddy(cBorder);
+	QWhatsThis::add( cBorder, i18n( "Use this combobox to change the border size of the decoration." ));
+	lBorder->hide();
+	cBorder->hide();
+	QHBoxLayout *borderSizeLayout = new QHBoxLayout(pluginSettingsGrp->layout() );
+	borderSizeLayout->addWidget(lBorder);
+	borderSizeLayout->addWidget(cBorder);
+	borderSizeLayout->addStretch();
+
 	pluginConfigWidget = new QVBox(pluginSettingsGrp);
 	pluginSettingsGrp->layout()->add( pluginConfigWidget );
 
 	// Page 2 (Button Selector)
-	QVBox* buttonPage = new QVBox( tabWidget );
-	buttonPage->setSpacing( KDialog::spacingHint() );
-	buttonPage->setMargin( KDialog::marginHint() );
+	QWidget* buttonPage = new QWidget( tabWidget );
+	QVBoxLayout* buttonLayout = new QVBoxLayout(buttonPage, KDialog::marginHint(), KDialog::spacingHint());
 
 	cbShowToolTips = new QCheckBox(
 			i18n("&Show window button tooltips"), buttonPage );
@@ -126,31 +126,32 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const char* name, co
 			i18n(  "Enabling this checkbox will show window button tooltips. "
 				   "If this checkbox is off, no window button tooltips will be shown."));
 
-        lBorder = new QLabel( buttonPage );
-        slBorder = new QSlider( Horizontal, buttonPage );
-        slBorder->setPageStep(1);
-        QWhatsThis::add( slBorder, i18n( "This slider shows all border sizes supported by this decoration." ));
-        lBorder->setBuddy( slBorder );
-        lBorder->hide();
-        slBorder->hide();
-
 	cbUseCustomButtonPositions = new QCheckBox(
 			i18n("Use custom titlebar button &positions"), buttonPage );
 	QWhatsThis::add( cbUseCustomButtonPositions,
-			i18n(  "The appropriate settings can be found in the \"Buttons\" Tab. "
-				   "Please note that this option is not available on all styles yet!" ) );
+			i18n(  "The appropriate settings can be found in the \"Buttons\" Tab; "
+				   "please note that this option is not available on all styles yet." ) );
 
-	buttonBox = new QGroupBox( 1, Qt::Horizontal,
-			i18n("Titlebar Button Positions"), buttonPage );
+	buttonLayout->addWidget( cbShowToolTips );
+	buttonLayout->addWidget( cbUseCustomButtonPositions );
 
 	// Add nifty dnd button modification widgets
-	QLabel* label = new QLabel( buttonBox );
-	dropSite = new ButtonDropSite( buttonBox );
-	label->setAlignment( int( QLabel::WordBreak ) );
-	label->setText( i18n( "To add or remove titlebar buttons, simply <i>drag</i> items "
-		"between the available item list and the titlebar preview. Similarly, "
-		"drag items within the titlebar preview to re-position them.") );
-	buttonSource = new ButtonSource( buttonBox );
+	buttonPositionWidget = new ButtonPositionWidget(buttonPage, "button_position_widget");
+	buttonPositionWidget->setDecorationFactory(plugins->factory() );
+	QHBoxLayout* buttonControlLayout = new QHBoxLayout(buttonLayout);
+	buttonControlLayout->addSpacing(20);
+	buttonControlLayout->addWidget(buttonPositionWidget);
+// 	buttonLayout->addStretch();
+
+	// preview
+	QVBoxLayout* previewLayout = new QVBoxLayout(layout, KDialog::spacingHint() );
+	previewLayout->setMargin( KDialog::marginHint() );
+
+	preview = new KDecorationPreview( this );
+	previewLayout->addWidget(preview);
+
+	preview->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+	tabWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
 
 	// Load all installed decorations into memory
 	// Set up the decoration lists and other UI settings
@@ -162,22 +163,28 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const char* name, co
 	tabWidget->insertTab( pluginPage, i18n("&Window Decoration") );
 	tabWidget->insertTab( buttonPage, i18n("&Buttons") );
 
-	connect( dropSite, SIGNAL(buttonAdded(char)), buttonSource, SLOT(hideButton(char)) );
-	connect( dropSite, SIGNAL(buttonRemoved(char)), buttonSource, SLOT(showButton(char)) );
-	connect( buttonSource, SIGNAL(buttonDropped()), dropSite, SLOT(removeClickedButton()) );
-	connect( dropSite, SIGNAL(changed()), this, SLOT(slotSelectionChanged()) );
-	connect( buttonSource, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()) );
+	connect( buttonPositionWidget, SIGNAL(changed()), this, SLOT(slotButtonsChanged()) ); // update preview etc.
+	connect( buttonPositionWidget, SIGNAL(changed()), this, SLOT(slotSelectionChanged()) ); // emit changed()...
 	connect( decorationList, SIGNAL(activated(const QString&)), SLOT(slotSelectionChanged()) );
 	connect( decorationList, SIGNAL(activated(const QString&)),
 								SLOT(slotChangeDecoration(const QString&)) );
 	connect( cbUseCustomButtonPositions, SIGNAL(clicked()), SLOT(slotSelectionChanged()) );
-	connect(cbUseCustomButtonPositions, SIGNAL(toggled(bool)), buttonBox, SLOT(setEnabled(bool)));
+	connect(cbUseCustomButtonPositions, SIGNAL(toggled(bool)), buttonPositionWidget, SLOT(setEnabled(bool)));
+	connect(cbUseCustomButtonPositions, SIGNAL(toggled(bool)), this, SLOT(slotButtonsChanged()) );
 	connect( cbShowToolTips, SIGNAL(clicked()), SLOT(slotSelectionChanged()) );
-        connect( slBorder, SIGNAL( valueChanged( int )), SLOT( slotBorderChanged( int )));
+	connect( cBorder, SIGNAL( activated( int )), SLOT( slotBorderChanged( int )));
 //	connect( cbUseMiniWindows, SIGNAL(clicked()), SLOT(slotSelectionChanged()) );
 
 	// Allow kwin dcop signal to update our selection list
 	connectDCOPSignal("kwin", 0, "dcopResetAllClients()", "dcopUpdateClientList()", false);
+	
+	KAboutData *about =
+		new KAboutData(I18N_NOOP("kcmkwindecoration"),
+				I18N_NOOP("Window Decoration Control Module"),
+				0, 0, KAboutData::License_GPL,
+				I18N_NOOP("(c) 2001 Karol Szwed"));
+	about->addAuthor("Karol Szwed", 0, "gallium@kde.org");
+	setAboutData(about);
 }
 
 
@@ -256,13 +263,13 @@ void KWinDecorationModule::slotSelectionChanged()
 
 static const char* const border_names[ KDecorationDefines::BordersCount ] =
     {
-    I18N_NOOP( "Border size: Tiny" ),
-    I18N_NOOP( "Border size: Normal" ),
-    I18N_NOOP( "Border size: Large" ),
-    I18N_NOOP( "Border size: Very Large" ),
-    I18N_NOOP( "Border size: Huge" ),
-    I18N_NOOP( "Border size: Very Huge" ),
-    I18N_NOOP( "Border size: Oversized" )
+    I18N_NOOP( "Tiny" ),
+    I18N_NOOP( "Normal" ),
+    I18N_NOOP( "Large" ),
+    I18N_NOOP( "Very Large" ),
+    I18N_NOOP( "Huge" ),
+    I18N_NOOP( "Very Huge" ),
+    I18N_NOOP( "Oversized" )
     };
 
 int KWinDecorationModule::borderSizeToIndex( BorderSize size, QValueList< BorderSize > sizes )
@@ -298,7 +305,15 @@ void KWinDecorationModule::slotBorderChanged( int size )
             sizes = plugins->factory()->borderSizes();
         assert( sizes.count() >= 2 );
         border_size = indexToBorderSize( size, sizes );
-        lBorder->setText( i18n( border_names[ border_size ] ));
+
+	// update preview
+	preview->setTempBorderSize(plugins, border_size);
+}
+
+void KWinDecorationModule::slotButtonsChanged()
+{
+	// update preview
+	preview->setTempButtons(plugins, cbUseCustomButtonPositions->isChecked(), buttonPositionWidget->buttonsLeft(), buttonPositionWidget->buttonsRight() );
 }
 
 QString KWinDecorationModule::decorationName( QString& libName )
@@ -362,6 +377,9 @@ void KWinDecorationModule::resetPlugin( KConfig* conf, const QString& currentDec
 
         checkSupportedBorderSizes();
 
+	// inform buttonPositionWidget about the new factory...
+	buttonPositionWidget->setDecorationFactory(plugins->factory() );
+
 	currentName = styleToConfigLib( currentName );
 
 	// Delete old plugin widget if it exists
@@ -390,12 +408,12 @@ void KWinDecorationModule::resetPlugin( KConfig* conf, const QString& currentDec
 			connect( this, SIGNAL(pluginLoad(KConfig*)), pluginObject, SLOT(load(KConfig*)) );
 			connect( this, SIGNAL(pluginSave(KConfig*)), pluginObject, SLOT(save(KConfig*)) );
 			connect( this, SIGNAL(pluginDefaults()), pluginObject, SLOT(defaults()) );
-			pluginSettingsGrp->show();
+			pluginConfigWidget->show();
 			return;
 		}
 	}
 
-	pluginSettingsGrp->hide();
+	pluginConfigWidget->hide();
 }
 
 
@@ -413,7 +431,7 @@ void KWinDecorationModule::readConfig( KConfig* conf )
 
 	oldLibraryName = currentLibraryName;
 	currentLibraryName = conf->readEntry("PluginLib",
-					((QPixmap::defaultDepth() > 8) ? "kwin_keramik" : "kwin_quartz"));
+					((QPixmap::defaultDepth() > 8) ? "kwin_plastik" : "kwin_quartz"));
 	QString decoName = decorationName( currentLibraryName );
 
 	// If we are using the "default" kde client, use the "default" entry.
@@ -434,21 +452,11 @@ void KWinDecorationModule::readConfig( KConfig* conf )
 	// ============
 	bool customPositions = conf->readBoolEntry("CustomButtonPositions", false);
 	cbUseCustomButtonPositions->setChecked( customPositions );
-	buttonBox->setEnabled( customPositions );
+	buttonPositionWidget->setEnabled( customPositions );
 	// Menu and onAllDesktops buttons are default on LHS
-	dropSite->buttonsLeft  = conf->readEntry("ButtonsOnLeft", "MS");
+	buttonPositionWidget->setButtonsLeft( conf->readEntry("ButtonsOnLeft", "MS") );
 	// Help, Minimize, Maximize and Close are default on RHS
-	dropSite->buttonsRight = conf->readEntry("ButtonsOnRight", "HIAX");
-	dropSite->repaint(false);
-
-	buttonSource->showAllButtons();
-
-	// Step through the button lists, and hide the dnd button source items
-	unsigned int i;
-	for(i = 0; i < dropSite->buttonsLeft.length(); i++)
-		buttonSource->hideButton( dropSite->buttonsLeft[i].latin1() );
-	for(i = 0; i < dropSite->buttonsRight.length(); i++)
-		buttonSource->hideButton( dropSite->buttonsRight[i].latin1() );
+	buttonPositionWidget->setButtonsRight( conf->readEntry("ButtonsOnRight", "HIAX") );
 
         int bsize = conf->readNumEntry( "BorderSize", BorderNormal );
         if( bsize >= BorderTiny && bsize < BordersCount )
@@ -477,8 +485,8 @@ void KWinDecorationModule::writeConfig( KConfig* conf )
 //	conf->writeEntry("MiniWindowBorders", cbUseMiniWindows->isChecked());
 
 	// Button settings
-	conf->writeEntry("ButtonsOnLeft", dropSite->buttonsLeft );
-	conf->writeEntry("ButtonsOnRight", dropSite->buttonsRight );
+	conf->writeEntry("ButtonsOnLeft", buttonPositionWidget->buttonsLeft() );
+	conf->writeEntry("ButtonsOnRight", buttonPositionWidget->buttonsRight() );
         conf->writeEntry("BorderSize", border_size );
 
 	oldLibraryName = currentLibraryName;
@@ -508,9 +516,8 @@ void KWinDecorationModule::load()
 	kwinConfig.setGroup("Style");
 
 	// Reset by re-reading the config
-	// The plugin doesn't need changing, as we have not saved
 	readConfig( &kwinConfig );
-	emit pluginLoad( &kwinConfig );
+        resetPlugin( &kwinConfig );
 }
 
 
@@ -532,24 +539,15 @@ void KWinDecorationModule::defaults()
 {
 	// Set the KDE defaults
 	cbUseCustomButtonPositions->setChecked( false );
-	buttonBox->setEnabled( false );
+	buttonPositionWidget->setEnabled( false );
 	cbShowToolTips->setChecked( true );
 //	cbUseMiniWindows->setChecked( false);
 // Don't set default for now
 //	decorationList->setSelected(
 //		decorationList->findItem( i18n("KDE 2") ), true );  // KDE classic client
 
-	dropSite->buttonsLeft = "MS";
-	dropSite->buttonsRight= "HIAX";
-	dropSite->repaint(false);
-
-	buttonSource->showAllButtons();
-	buttonSource->hideButton('M');
-	buttonSource->hideButton('S');
-	buttonSource->hideButton('H');
-	buttonSource->hideButton('I');
-	buttonSource->hideButton('A');
-	buttonSource->hideButton('X');
+	buttonPositionWidget->setButtonsLeft("MS");
+	buttonPositionWidget->setButtonsRight("HIAX");
 
         border_size = BorderNormal;
         checkSupportedBorderSizes();
@@ -561,18 +559,23 @@ void KWinDecorationModule::defaults()
 void KWinDecorationModule::checkSupportedBorderSizes()
 {
         QValueList< BorderSize > sizes;
-        slBorder->hide();
-        lBorder->hide();
         if( plugins->factory() != NULL )
             sizes = plugins->factory()->borderSizes();
-        if( sizes.count() < 2 )
-            return;
-        slBorder->setRange( 0, sizes.count() - 1 );
-        int pos = borderSizeToIndex( border_size, sizes );
-        lBorder->show();
-        slBorder->show();
-        slBorder->setValue( pos );
-        slotBorderChanged( pos );
+	if( sizes.count() < 2 ) {
+		lBorder->hide();
+		cBorder->hide();
+	} else {
+		cBorder->clear();
+		for (QValueList<BorderSize>::const_iterator it = sizes.begin(); it != sizes.end(); ++it) {
+			BorderSize size = *it;
+			cBorder->insertItem(i18n(border_names[size]), borderSizeToIndex(size,sizes) );
+		}
+		int pos = borderSizeToIndex( border_size, sizes );
+		lBorder->show();
+		cBorder->show();
+		cBorder->setCurrentItem(pos);
+		slotBorderChanged( pos );
+	}
 }
 
 QString KWinDecorationModule::styleToConfigLib( QString& styleLib )
@@ -589,28 +592,16 @@ QString KWinDecorationModule::quickHelp() const
 		"<p>This module allows you to choose the window border decorations, "
 		"as well as titlebar button positions and custom decoration options.</p>"
 		"To choose a theme for your window decoration click on its name and apply your choice by clicking the \"Apply\" button below."
-		" If you don't want to apply your choice you can click the \"Reset\" button to discard your changes."
+		" If you do not want to apply your choice you can click the \"Reset\" button to discard your changes."
 		"<p>You can configure each theme in the \"Configure [...]\" tab. There are different options specific for each theme.</p>"
 		"<p>In \"General Options (if available)\" you can activate the \"Buttons\" tab by checking the \"Use custom titlebar button positions\" box."
 		" In the \"Buttons\" tab you can change the positions of the buttons to your liking.</p>" );
 }
 
 
-const KAboutData* KWinDecorationModule::aboutData() const
-{
-	KAboutData* about =
-		new KAboutData(I18N_NOOP("kcmkwindecoration"),
-				I18N_NOOP("Window Decoration Control Module"),
-				0, 0, KAboutData::License_GPL,
-				I18N_NOOP("(c) 2001 Karol Szwed"));
-	about->addAuthor("Karol Szwed", 0, "gallium@kde.org");
-	return about;
-}
-
-
 void KWinDecorationModule::resetKWin()
 {
-	bool ok = kapp->dcopClient()->send("kwin", "KWinInterface",
+	bool ok = kapp->dcopClient()->send("kwin*", "KWinInterface",
                         "reconfigure()", QByteArray());
 	if (!ok)
 		kdDebug() << "kcmkwindecoration: Could not reconfigure kwin" << endl;
@@ -618,4 +609,5 @@ void KWinDecorationModule::resetKWin()
 
 #include "kwindecoration.moc"
 // vim: ts=4
+// kate: space-indent off; tab-width 4;
 
