@@ -21,6 +21,10 @@ License. See the file "COPYING" for the exact licensing terms.
 #include "workspace.h"
 
 #include <fixx11h.h>
+#include <qhbox.h>
+#include <qpushbutton.h>
+#include <qslider.h>
+#include <qtooltip.h>
 #include <qpopupmenu.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
@@ -28,8 +32,8 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <kconfig.h>
 #include <kglobalaccel.h>
 #include <kapplication.h>
+#include <qregexp.h>
 
-#include "popupinfo.h"
 #include "killwindow.h"
 #include "tabbox.h"
 
@@ -49,7 +53,7 @@ QPopupMenu* Workspace::clientPopup()
         popup->setFont(KGlobalSettings::menuFont());
         connect( popup, SIGNAL( aboutToShow() ), this, SLOT( clientPopupAboutToShow() ) );
         connect( popup, SIGNAL( activated(int) ), this, SLOT( clientPopupActivated(int) ) );
-
+      
         advanced_popup = new QPopupMenu( popup );
         advanced_popup->setCheckable( TRUE );
         advanced_popup->setFont(KGlobalSettings::menuFont());
@@ -61,10 +65,30 @@ QPopupMenu* Workspace::clientPopup()
         advanced_popup->insertItem( SmallIconSet( "window_fullscreen" ),
             i18n("&Fullscreen")+'\t'+keys->shortcut("Window Fullscreen").seq(0).toString(), Options::FullScreenOp );
         advanced_popup->insertItem( i18n("&No Border")+'\t'+keys->shortcut("Window No Border").seq(0).toString(), Options::NoBorderOp );
-        advanced_popup->insertItem( SmallIconSet( "filesave" ), i18n("&Special Window Settings..."), Options::WindowRulesOp );
+        advanced_popup->insertItem( SmallIconSet("key_bindings"),
+            i18n("Window &Shortcut...")+'\t'+keys->shortcut("Setup Window Shortcut").seq(0).toString(), Options::SetupWindowShortcutOp );
+        advanced_popup->insertItem( SmallIconSet( "wizard" ), i18n("&Special Window Settings..."), Options::WindowRulesOp );
+        advanced_popup->insertItem( SmallIconSet( "wizard" ), i18n("&Special Application Settings..."), Options::ApplicationRulesOp );
 
         popup->insertItem(i18n("Ad&vanced"), advanced_popup );
         desk_popup_index = popup->count();
+        
+        if (options->useTranslucency){
+            QPopupMenu *trans_popup = new QPopupMenu( popup );
+            QVBox *transBox = new QVBox(trans_popup);
+            transButton = new QPushButton(transBox, "transButton");
+            QToolTip::add(transButton, i18n("Reset opacity to default value"));
+            transSlider = new QSlider(0, 100, 1, 100, Qt::Vertical, transBox, "transSlider");
+            QToolTip::add(transSlider, i18n("Slide this to set the window's opacity"));
+            connect(transButton, SIGNAL(clicked()), SLOT(resetClientOpacity()));
+            connect(transButton, SIGNAL(clicked()), trans_popup, SLOT(hide()));
+            connect(transSlider, SIGNAL(valueChanged(int)), SLOT(setTransButtonText(int)));
+            connect(transSlider, SIGNAL(valueChanged(int)), this, SLOT(setPopupClientOpacity(int)));
+//             connect(transSlider, SIGNAL(sliderReleased()), trans_popup, SLOT(hide()));
+            trans_popup->insertItem(transBox);
+            popup->insertItem(i18n("&Opacity"), trans_popup );
+        }
+        
         popup->insertItem( SmallIconSet( "move" ), i18n("&Move")+'\t'+keys->shortcut("Window Move").seq(0).toString(), Options::MoveOp );
         popup->insertItem( i18n("Re&size")+'\t'+keys->shortcut("Window Resize").seq(0).toString(), Options::ResizeOp );
         popup->insertItem( i18n("Mi&nimize")+'\t'+keys->shortcut("Window Minimize").seq(0).toString(), Options::MinimizeOp );
@@ -84,6 +108,36 @@ QPopupMenu* Workspace::clientPopup()
         }
     return popup;
     }
+    
+//sets the transparency of the client to given value(given by slider)
+void Workspace::setPopupClientOpacity(int value)
+    {
+    active_popup_client->setCustomOpacityFlag(true);
+    value = 100 - value;
+    value<100?active_popup_client->setOpacity(true, (uint)((value/100.0)*0xffffffff)):active_popup_client->setOpacity(false,0xffffffff);
+    }
+
+void Workspace::setTransButtonText(int value)
+    {
+    value = 100 - value;
+    if(value < 0)
+        transButton->setText("000 %");
+    else if (value >= 100 )
+        transButton->setText("100 %");
+    else if(value < 10)
+        transButton->setText("00"+QString::number(value)+" %");
+    else if(value < 100)
+        transButton->setText("0"+QString::number(value)+" %");
+    }
+
+void Workspace::resetClientOpacity()
+    {
+    active_popup_client->setCustomOpacityFlag(false);
+    active_popup_client->updateOpacity();
+    transSlider->setValue(100-active_popup_client->opacityPercentage());
+    setTransButtonText(100-active_popup_client->opacityPercentage());
+    }
+
 
 /*!
   The client popup menu will become visible soon.
@@ -92,7 +146,7 @@ QPopupMenu* Workspace::clientPopup()
  */
 void Workspace::clientPopupAboutToShow()
     {
-    if ( !popup_client || !popup )
+    if ( !active_popup_client || !popup )
         return;
 
     if ( numberOfDesktops() == 1 )
@@ -105,21 +159,26 @@ void Workspace::clientPopupAboutToShow()
         initDesktopPopup();
         }
 
-    popup->setItemEnabled( Options::ResizeOp, popup_client->isResizable() );
-    popup->setItemEnabled( Options::MoveOp, popup_client->isMovable() );
-    popup->setItemEnabled( Options::MaximizeOp, popup_client->isMaximizable() );
-    popup->setItemChecked( Options::MaximizeOp, popup_client->maximizeMode() == Client::MaximizeFull );
+    popup->setItemEnabled( Options::ResizeOp, active_popup_client->isResizable() );
+    popup->setItemEnabled( Options::MoveOp, active_popup_client->isMovable() );
+    popup->setItemEnabled( Options::MaximizeOp, active_popup_client->isMaximizable() );
+    popup->setItemChecked( Options::MaximizeOp, active_popup_client->maximizeMode() == Client::MaximizeFull );
     // This should be checked also when hover unshaded
-    popup->setItemChecked( Options::ShadeOp, popup_client->shadeMode() != ShadeNone );
-    popup->setItemEnabled( Options::ShadeOp, popup_client->isShadeable());
-    advanced_popup->setItemChecked( Options::KeepAboveOp, popup_client->keepAbove() );
-    advanced_popup->setItemChecked( Options::KeepBelowOp, popup_client->keepBelow() );
-    advanced_popup->setItemChecked( Options::FullScreenOp, popup_client->isFullScreen() );
-    advanced_popup->setItemEnabled( Options::FullScreenOp, popup_client->userCanSetFullScreen() );
-    advanced_popup->setItemChecked( Options::NoBorderOp, popup_client->noBorder() );
-    advanced_popup->setItemEnabled( Options::NoBorderOp, popup_client->userCanSetNoBorder() );
-    popup->setItemEnabled( Options::MinimizeOp, popup_client->isMinimizable() );
-    popup->setItemEnabled( Options::CloseOp, popup_client->isCloseable() );
+    popup->setItemChecked( Options::ShadeOp, active_popup_client->shadeMode() != ShadeNone );
+    popup->setItemEnabled( Options::ShadeOp, active_popup_client->isShadeable());
+    advanced_popup->setItemChecked( Options::KeepAboveOp, active_popup_client->keepAbove() );
+    advanced_popup->setItemChecked( Options::KeepBelowOp, active_popup_client->keepBelow() );
+    advanced_popup->setItemChecked( Options::FullScreenOp, active_popup_client->isFullScreen() );
+    advanced_popup->setItemEnabled( Options::FullScreenOp, active_popup_client->userCanSetFullScreen() );
+    advanced_popup->setItemChecked( Options::NoBorderOp, active_popup_client->noBorder() );
+    advanced_popup->setItemEnabled( Options::NoBorderOp, active_popup_client->userCanSetNoBorder() );
+    popup->setItemEnabled( Options::MinimizeOp, active_popup_client->isMinimizable() );
+    popup->setItemEnabled( Options::CloseOp, active_popup_client->isCloseable() );
+    if (options->useTranslucency)
+        {
+        transSlider->setValue(100-active_popup_client->opacityPercentage());
+        setTransButtonText(100-active_popup_client->opacityPercentage());
+        }
     }
 
 
@@ -150,7 +209,7 @@ void Workspace::desktopPopupAboutToShow()
 
     desk_popup->clear();
     desk_popup->insertItem( i18n("&All Desktops"), 0 );
-    if ( active_client && active_client->isOnAllDesktops() )
+    if ( active_popup_client && active_popup_client->isOnAllDesktops() )
         desk_popup->setItemChecked( 0, TRUE );
     desk_popup->insertSeparator( -1 );
     int id;
@@ -167,13 +226,21 @@ void Workspace::desktopPopupAboutToShow()
                     .arg(i)
                     .arg( desktopName(i).replace( '&', "&&" )),
                 i );
-        if ( active_client &&
-             !active_client->isOnAllDesktops() && active_client->desktop()  == i )
+        if ( active_popup_client &&
+             !active_popup_client->isOnAllDesktops() && active_popup_client->desktop()  == i )
             desk_popup->setItemChecked( id, TRUE );
         }
     }
 
-
+void Workspace::closeActivePopup()
+    {
+    if( active_popup )
+        {
+        active_popup->close();
+        active_popup = NULL;
+        active_popup_client = NULL;
+        }
+    }
 
 /*!
   Create the global accel object \c keys.
@@ -181,6 +248,11 @@ void Workspace::desktopPopupAboutToShow()
 void Workspace::initShortcuts()
     {
     keys = new KGlobalAccel( this );
+    // a separate KGlobalAccel is needed for the shortcut for disabling global shortcuts,
+    // otherwise it would also disable itself
+    disable_shortcuts_keys = new KGlobalAccel( this );
+    disable_shortcuts_keys->disableBlocking( true );
+#define IN_KWIN
 #include "kwinbindings.cpp"
     readShortcuts();
     }
@@ -188,6 +260,7 @@ void Workspace::initShortcuts()
 void Workspace::readShortcuts()
     {
     keys->readSettings();
+    disable_shortcuts_keys->readSettings();
 
     cutWalkThroughDesktops = keys->shortcut("Walk Through Desktops");
     cutWalkThroughDesktopsReverse = keys->shortcut("Walk Through Desktops (Reverse)");
@@ -197,6 +270,7 @@ void Workspace::readShortcuts()
     cutWalkThroughWindowsReverse = keys->shortcut("Walk Through Windows (Reverse)");
 
     keys->updateConnections();
+    disable_shortcuts_keys->updateConnections();
     
     delete popup;
     popup = NULL; // so that it's recreated next time
@@ -204,10 +278,61 @@ void Workspace::readShortcuts()
     }
 
 
+void Workspace::setupWindowShortcut( Client* c )
+    {
+    assert( client_keys_dialog == NULL );
+    keys->suspend( true );
+    disable_shortcuts_keys->suspend( true );
+    client_keys->suspend( true );
+    client_keys_dialog = new ShortcutDialog( c->shortcut());
+    client_keys_client = c;
+    connect( client_keys_dialog, SIGNAL( dialogDone( bool )), SLOT( setupWindowShortcutDone( bool )));
+    QRect r = clientArea( ScreenArea, c );
+    QSize size = client_keys_dialog->sizeHint();
+    QPoint pos = c->pos() + c->clientPos();
+    if( pos.x() + size.width() >= r.right())
+        pos.setX( r.right() - size.width());
+    if( pos.y() + size.height() >= r.bottom())
+        pos.setY( r.bottom() - size.height());
+    client_keys_dialog->move( pos );
+    client_keys_dialog->show();
+    active_popup = client_keys_dialog;
+    active_popup_client = c;
+    }
+
+void Workspace::setupWindowShortcutDone( bool ok )
+    {
+    keys->suspend( false );
+    disable_shortcuts_keys->suspend( false );
+    client_keys->suspend( false );
+    if( ok )
+        {
+        client_keys_client->setShortcut( KShortcut( client_keys_dialog->shortcut()).toString());
+        }
+    closeActivePopup();
+    delete client_keys_dialog;
+    client_keys_dialog = NULL;
+    client_keys_client = NULL;
+    }
+
+void Workspace::clientShortcutUpdated( Client* c )
+    {
+    QString key = QString::number( c->window());
+    client_keys->remove( key );
+    if( !c->shortcut().isNull())
+        {
+        client_keys->insert( key, key );
+        client_keys->setShortcut( key, c->shortcut());
+        client_keys->setSlot( key, c, SLOT( shortcutActivated()));
+        client_keys->setActionEnabled( key, true );
+        }
+    client_keys->updateConnections();
+    }
+
 void Workspace::clientPopupActivated( int id )
     {
     WindowOperation op = static_cast< WindowOperation >( id );
-    Client* c = popup_client ? popup_client : active_client;
+    Client* c = active_popup_client ? active_popup_client : active_client;
     QString type;
     switch( op )
         {
@@ -264,11 +389,14 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
         case Options::VMaximizeOp:
             c->maximize( c->maximizeMode() ^ Client::MaximizeVertical );
             break;
+        case Options::RestoreOp:
+            c->maximize( Client::MaximizeRestore );
+            break;
         case Options::MinimizeOp:
             c->minimize();
             break;
         case Options::ShadeOp:
-            c->toggleShade();
+            c->performMouseCommand( Options::MouseShade, QCursor::pos());
             break;
         case Options::OnAllDesktopsOp:
             c->setOnAllDesktops( !c->isOnAllDesktops() );
@@ -280,18 +408,39 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
             c->setUserNoBorder( !c->isUserNoBorder());
             break;
         case Options::KeepAboveOp:
+            {
+            StackingUpdatesBlocker blocker( this );
+            bool was = c->keepAbove();
             c->setKeepAbove( !c->keepAbove() );
+            if( was && !c->keepAbove())
+                raiseClient( c );
             break;
+            }
         case Options::KeepBelowOp:
+            {
+            StackingUpdatesBlocker blocker( this );
+            bool was = c->keepBelow();
             c->setKeepBelow( !c->keepBelow() );
+            if( was && !c->keepBelow())
+                lowerClient( c );
+            break;
+            }
+        case Options::OperationsOp:
+            c->performMouseCommand( Options::MouseShade, QCursor::pos());
             break;
         case Options::WindowRulesOp:
-            editWindowRules( c );
+            editWindowRules( c, false );
+            break;
+        case Options::ApplicationRulesOp:
+            editWindowRules( c, true );
+            break;
+        case Options::SetupWindowShortcutOp:
+            setupWindowShortcut( c );
             break;
         case Options::LowerOp:
             lowerClient(c);
             break;
-        default:
+        case Options::NoOp:
             break;
         }
     }
@@ -312,9 +461,18 @@ bool Client::performMouseCommand( Options::MouseCommand command, QPoint globalPo
             break;
         case Options::MouseShade :
             toggleShade();
+            cancelShadeHover();
+            break;
+        case Options::MouseSetShade:
+            setShade( ShadeNormal );
+            cancelShadeHover();
+            break;
+        case Options::MouseUnsetShade:
+            setShade( ShadeNone );
+            cancelShadeHover();
             break;
         case Options::MouseOperationsMenu:
-            if ( isActive() & options->clickRaise )
+            if ( isActive() && options->clickRaise )
                 autoRaise();
             workspace()->showWindowMenu( globalPos, this );
             break;
@@ -372,7 +530,7 @@ bool Client::performMouseCommand( Options::MouseCommand command, QPoint globalPo
         case Options::MouseResize:
         case Options::MouseUnrestrictedResize:
             {
-            if (!isResizable() || isShade()) // SHADE
+            if (!isResizable() || isShade())
                 break;
             if( moveResizeMode )
                 finishMoveResize( false );
@@ -399,12 +557,62 @@ bool Client::performMouseCommand( Options::MouseCommand command, QPoint globalPo
                 }
             break;
             }
+        case Options::MouseMaximize:
+            maximize( Client::MaximizeFull );
+            break;
+        case Options::MouseRestore:
+            maximize( Client::MaximizeRestore );
+            break;
         case Options::MouseMinimize:
             minimize();
             break;
+        case Options::MouseAbove:
+            {
+            StackingUpdatesBlocker blocker( workspace());
+            if( keepBelow())
+                setKeepBelow( false );
+            else
+                setKeepAbove( true );
+            break;
+            }
+        case Options::MouseBelow:
+            {
+            StackingUpdatesBlocker blocker( workspace());
+            if( keepAbove())
+                setKeepAbove( false );
+            else
+                setKeepBelow( true );
+            break;
+            }
+        case Options::MousePreviousDesktop:
+            workspace()->windowToPreviousDesktop( this );
+            break;
+        case Options::MouseNextDesktop:
+            workspace()->windowToNextDesktop( this );
+            break;
+        case Options::MouseOpacityMore:
+            if (opacity_ < 0xFFFFFFFF)
+                {
+                if (opacity_ < 0xF3333333)
+                    {
+                    setOpacity(TRUE, opacity_ + 0xCCCCCCC);
+                    custom_opacity = true;
+                    }
+                else
+                    {
+                    setOpacity(FALSE, 0xFFFFFFFF);
+                    custom_opacity = false;
+                    }
+                }
+            break;
+        case Options::MouseOpacityLess:
+            if (opacity_ > 0)
+                {
+                setOpacity(TRUE, (opacity_ > 0xCCCCCCC) ? opacity_ - 0xCCCCCCC : 0);
+                custom_opacity = true;
+                }
+            break;
         case Options::MouseNothing:
-        // fall through
-        default:
             replay = TRUE;
             break;
         }
@@ -438,7 +646,6 @@ void Workspace::slotSwitchDesktopNext()
             }
         }
     setCurrentDesktop(d);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
     }
 
 void Workspace::slotSwitchDesktopPrevious()
@@ -452,151 +659,54 @@ void Workspace::slotSwitchDesktopPrevious()
           return;
         }
     setCurrentDesktop(d);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
     }
 
 void Workspace::slotSwitchDesktopRight()
     {
-    int x,y;
-    calcDesktopLayout(x,y);
-    int dt = currentDesktop()-1;
-    if (layoutOrientation == Qt::Vertical)
-        {
-        dt += y;
-        if ( dt >= numberOfDesktops() ) 
-            {
-            if ( options->rollOverDesktops )
-              dt -= numberOfDesktops();
-            else
-              return;
-            }
-        }
-    else
-        {
-        int d = (dt % x) + 1;
-        if ( d >= x ) 
-            {
-            if ( options->rollOverDesktops )
-              d -= x;
-            else
-              return;
-            }
-        dt = dt - (dt % x) + d;
-        }
-    setCurrentDesktop(dt+1);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
+    int desktop = desktopToRight( currentDesktop());
+    if( desktop == currentDesktop())
+        return;
+    setCurrentDesktop( desktop );
     }
 
 void Workspace::slotSwitchDesktopLeft()
     {
-    int x,y;
-    calcDesktopLayout(x,y);
-    int dt = currentDesktop()-1;
-    if (layoutOrientation == Qt::Vertical)
-        {
-        dt -= y;
-        if ( dt < 0 ) 
-            {
-            if ( options->rollOverDesktops )
-              dt += numberOfDesktops();
-            else
-              return;
-            }
-        }
-    else
-        {
-        int d = (dt % x) - 1;
-        if ( d < 0 ) 
-            {
-            if ( options->rollOverDesktops )
-              d += x;
-            else
-              return;
-            }
-        dt = dt - (dt % x) + d;
-        }
-    setCurrentDesktop(dt+1);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
+    int desktop = desktopToLeft( currentDesktop());
+    if( desktop == currentDesktop())
+        return;
+    setCurrentDesktop( desktop );
     }
 
 void Workspace::slotSwitchDesktopUp()
     {
-    int x,y;
-    calcDesktopLayout(x,y);
-    int dt = currentDesktop()-1;
-    if (layoutOrientation == Qt::Horizontal)
-        {
-        dt -= x;
-        if ( dt < 0 ) 
-            {
-            if ( options->rollOverDesktops )
-              dt += numberOfDesktops();
-            else
-              return;
-            }
-        }
-    else
-        {
-        int d = (dt % y) - 1;
-        if ( d < 0 ) 
-            {
-            if ( options->rollOverDesktops )
-              d += y;
-            else
-              return;
-            }
-        dt = dt - (dt % y) + d;
-        }
-    setCurrentDesktop(dt+1);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
+    int desktop = desktopUp( currentDesktop());
+    if( desktop == currentDesktop())
+        return;
+    setCurrentDesktop( desktop );
     }
 
 void Workspace::slotSwitchDesktopDown()
     {
-    int x,y;
-    calcDesktopLayout(x,y);
-    int dt = currentDesktop()-1;
-    if (layoutOrientation == Qt::Horizontal)
-        {
-        dt += x;
-        if ( dt >= numberOfDesktops() ) 
-            {
-            if ( options->rollOverDesktops )
-              dt -= numberOfDesktops();
-            else
-              return;
-            }
-        }
-    else
-        {
-        int d = (dt % y) + 1;
-        if ( d >= y ) 
-            {
-            if ( options->rollOverDesktops )
-              d -= y;
-            else
-              return;
-            }
-        dt = dt - (dt % y) + d;
-        }
-    setCurrentDesktop(dt+1);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
+    int desktop = desktopDown( currentDesktop());
+    if( desktop == currentDesktop())
+        return;
+    setCurrentDesktop( desktop );
     }
 
 void Workspace::slotSwitchToDesktop( int i )
     {
     setCurrentDesktop( i );
-    popupinfo->showInfo( desktopName(currentDesktop()) );
     }
 
 
 void Workspace::slotWindowToDesktop( int i )
     {
-    if( i >= 1 && i <= numberOfDesktops() && active_client
-        && !active_client->isDesktop()
-        && !active_client->isDock()
-        && !active_client->isTopMenu())
-            sendClientToDesktop( active_client, i, true );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( i >= 1 && i <= numberOfDesktops() && c
+        && !c->isDesktop()
+        && !c->isDock()
+        && !c->isTopMenu())
+            sendClientToDesktop( c, i, true );
     }
 
 /*!
@@ -604,8 +714,9 @@ void Workspace::slotWindowToDesktop( int i )
  */
 void Workspace::slotWindowMaximize()
     {
-    if ( active_client )
-        performWindowOperation( active_client, Options::MaximizeOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if ( c )
+        performWindowOperation( c, Options::MaximizeOp );
     }
 
 /*!
@@ -613,8 +724,9 @@ void Workspace::slotWindowMaximize()
  */
 void Workspace::slotWindowMaximizeVertical()
     {
-    if ( active_client )
-        performWindowOperation( active_client, Options::VMaximizeOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if ( c )
+        performWindowOperation( c, Options::VMaximizeOp );
     }
 
 /*!
@@ -622,8 +734,9 @@ void Workspace::slotWindowMaximizeVertical()
  */
 void Workspace::slotWindowMaximizeHorizontal()
     {
-    if ( active_client )
-        performWindowOperation( active_client, Options::HMaximizeOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if ( c )
+        performWindowOperation( c, Options::HMaximizeOp );
     }
 
 
@@ -632,7 +745,8 @@ void Workspace::slotWindowMaximizeHorizontal()
  */
 void Workspace::slotWindowMinimize()
     {
-    performWindowOperation( active_client, Options::MinimizeOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    performWindowOperation( c, Options::MinimizeOp );
     }
 
 /*!
@@ -640,7 +754,8 @@ void Workspace::slotWindowMinimize()
  */
 void Workspace::slotWindowShade()
     {
-    performWindowOperation( active_client, Options::ShadeOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    performWindowOperation( c, Options::ShadeOp );
     }
 
 /*!
@@ -648,8 +763,9 @@ void Workspace::slotWindowShade()
  */
 void Workspace::slotWindowRaise()
     {
-    if ( active_client )
-        raiseClient( active_client );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if ( c )
+        raiseClient( c );
     }
 
 /*!
@@ -657,8 +773,9 @@ void Workspace::slotWindowRaise()
  */
 void Workspace::slotWindowLower()
     {
-    if ( active_client )
-        lowerClient( active_client );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if ( c )
+        lowerClient( c );
     }
 
 /*!
@@ -666,55 +783,71 @@ void Workspace::slotWindowLower()
   */
 void Workspace::slotWindowRaiseOrLower()
     {
-    if  ( active_client )
-        raiseOrLowerClient( active_client );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if  ( c )
+        raiseOrLowerClient( c );
     }
 
 void Workspace::slotWindowOnAllDesktops()
     {
-    if( active_client )
-        active_client->setOnAllDesktops( !active_client->isOnAllDesktops());
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( c )
+        c->setOnAllDesktops( !c->isOnAllDesktops());
     }
 
 void Workspace::slotWindowFullScreen()
     {
-    if( active_client )
-        performWindowOperation( active_client, Options::FullScreenOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( c )
+        performWindowOperation( c, Options::FullScreenOp );
     }
 
 void Workspace::slotWindowNoBorder()
     {
-    if( active_client )
-        performWindowOperation( active_client, Options::NoBorderOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( c )
+        performWindowOperation( c, Options::NoBorderOp );
     }
 
 void Workspace::slotWindowAbove()
     {
-    if( active_client )
-        performWindowOperation( active_client, Options::KeepAboveOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( c )
+        performWindowOperation( c, Options::KeepAboveOp );
     }
 
 void Workspace::slotWindowBelow()
     {
-    if( active_client )
-        performWindowOperation( active_client, Options::KeepBelowOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( c )
+        performWindowOperation( c, Options::KeepBelowOp );
+    }
+void Workspace::slotSetupWindowShortcut()
+    {
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if( c )
+        performWindowOperation( c, Options::SetupWindowShortcutOp );
     }
 
 /*!
   Move window to next desktop
  */
 void Workspace::slotWindowToNextDesktop()
+    {
+    windowToNextDesktop( active_popup_client ? active_popup_client : active_client );
+    }
+    
+void Workspace::windowToNextDesktop( Client* c )
     {
     int d = currentDesktop() + 1;
     if ( d > numberOfDesktops() )
         d = 1;
-    if (active_client && !active_client->isDesktop()
-        && !active_client->isDock() && !active_client->isTopMenu())
+    if (c && !c->isDesktop()
+        && !c->isDock() && !c->isTopMenu())
         {
-        setClientIsMoving( active_client );
+        setClientIsMoving( c );
         setCurrentDesktop( d );
         setClientIsMoving( NULL );
-        popupinfo->showInfo( desktopName(currentDesktop()) );
         }
     }
 
@@ -722,20 +855,85 @@ void Workspace::slotWindowToNextDesktop()
   Move window to previous desktop
  */
 void Workspace::slotWindowToPreviousDesktop()
+    {
+    windowToPreviousDesktop( active_popup_client ? active_popup_client : active_client );
+    }
+    
+void Workspace::windowToPreviousDesktop( Client* c )
     {
     int d = currentDesktop() - 1;
     if ( d <= 0 )
         d = numberOfDesktops();
-    if (active_client && !active_client->isDesktop()
-        && !active_client->isDock() && !active_client->isTopMenu())
+    if (c && !c->isDesktop()
+        && !c->isDock() && !c->isTopMenu())
         {
-        setClientIsMoving( active_client );
+        setClientIsMoving( c );
         setCurrentDesktop( d );
         setClientIsMoving( NULL );
-        popupinfo->showInfo( desktopName(currentDesktop()) );
         }
     }
 
+void Workspace::slotWindowToDesktopRight()
+    {
+    int d = desktopToRight( currentDesktop());
+    if( d == currentDesktop())
+        return;
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if (c && !c->isDesktop()
+        && !c->isDock() && !c->isTopMenu())
+        {
+        setClientIsMoving( c );
+        setCurrentDesktop( d );
+        setClientIsMoving( NULL );
+        }
+    }
+
+void Workspace::slotWindowToDesktopLeft()
+    {
+    int d = desktopToLeft( currentDesktop());
+    if( d == currentDesktop())
+        return;
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if (c && !c->isDesktop()
+        && !c->isDock() && !c->isTopMenu())
+        {
+        setClientIsMoving( c );
+        setCurrentDesktop( d );
+        setClientIsMoving( NULL );
+        }
+    }
+
+void Workspace::slotWindowToDesktopUp()
+    {
+    int d = desktopUp( currentDesktop());
+    if( d == currentDesktop())
+        return;
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if (c && !c->isDesktop()
+        && !c->isDock() && !c->isTopMenu())
+        {
+        setClientIsMoving( c );
+        setCurrentDesktop( d );
+        setClientIsMoving( NULL );
+        }
+    }
+
+void Workspace::slotWindowToDesktopDown()
+    {
+    int d = desktopDown( currentDesktop());
+    if( d == currentDesktop())
+        return;
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    if (c && !c->isDesktop()
+        && !c->isDock() && !c->isTopMenu())
+        {
+        setClientIsMoving( c );
+        setCurrentDesktop( d );
+        setClientIsMoving( NULL );
+        }
+    }
+
+
 /*!
   Kill Window feature, similar to xkill
  */
@@ -752,15 +950,15 @@ void Workspace::slotKillWindow()
  */
 void Workspace::slotSendToDesktop( int desk )
     {
-    if ( !popup_client )
+    if ( !active_popup_client )
         return;
     if ( desk == 0 ) 
         { // the 'on_all_desktops' menu entry
-        popup_client->setOnAllDesktops( !popup_client->isOnAllDesktops());
+        active_popup_client->setOnAllDesktops( !active_popup_client->isOnAllDesktops());
         return;
         }
 
-    sendClientToDesktop( popup_client, desk, false );
+    sendClientToDesktop( active_popup_client, desk, false );
 
     }
 
@@ -781,29 +979,33 @@ void Workspace::showWindowMenu( const QRect &pos, Client* cl )
         return;
     if( !cl )
         return;
-    if( popup_client != NULL ) // recursion
+    if( active_popup_client != NULL ) // recursion
         return;
     if ( cl->isDesktop()
         || cl->isDock()
         || cl->isTopMenu())
         return;
 
-    popup_client = cl;
+    active_popup_client = cl;
     QPopupMenu* p = clientPopup();
+    active_popup = p;
     int x = pos.left();
     int y = pos.bottom();
-    if (y == pos.top()) {
+    if (y == pos.top())
 	p->exec( QPoint( x, y ) );
-    } else {
+    else
+        {
 	QRect area = clientArea(ScreenArea, QPoint(x, y), currentDesktop());
+        clientPopupAboutToShow(); // needed for sizeHint() to be correct :-/
 	int popupHeight = p->sizeHint().height();
-	if (y + popupHeight < area.height()) {
+	if (y + popupHeight < area.height())
 	    p->exec( QPoint( x, y ) );
-	} else {
+	else
 	    p->exec( QPoint( x, pos.top() - popupHeight ) );
-	}
-    }
-    popup_client = 0;
+        }
+    // active popup may be already changed (e.g. the window shortcut dialog)
+    if( active_popup == p )
+        closeActivePopup();
     }
 
 /*!
@@ -811,9 +1013,10 @@ void Workspace::showWindowMenu( const QRect &pos, Client* cl )
  */
 void Workspace::slotWindowClose()
     {
-    if ( tab_box->isVisible() || popupinfo->isVisible() )
+    if ( tab_box->isVisible())
         return;
-    performWindowOperation( active_client, Options::CloseOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    performWindowOperation( c, Options::CloseOp );
     }
 
 /*!
@@ -821,7 +1024,8 @@ void Workspace::slotWindowClose()
  */
 void Workspace::slotWindowMove()
     {
-    performWindowOperation( active_client, Options::UnrestrictedMoveOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    performWindowOperation( c, Options::UnrestrictedMoveOp );
     }
 
 /*!
@@ -829,7 +1033,87 @@ void Workspace::slotWindowMove()
  */
 void Workspace::slotWindowResize()
     {
-    performWindowOperation( active_client, Options::UnrestrictedResizeOp );
+    Client* c = active_popup_client ? active_popup_client : active_client;
+    performWindowOperation( c, Options::UnrestrictedResizeOp );
+    }
+
+void Client::setShortcut( const QString& _cut )
+    {
+    QString cut = rules()->checkShortcut( _cut );
+    if( cut.isEmpty())
+        return setShortcutInternal( KShortcut());
+// Format:
+// base+(abcdef)<space>base+(abcdef)
+// E.g. Alt+Ctrl+(ABCDEF) Win+X,Win+(ABCDEF)
+    if( !cut.contains( '(' ) && !cut.contains( ')' ) && !cut.contains( ' ' ))
+        {
+        if( workspace()->shortcutAvailable( KShortcut( cut ), this ))
+            setShortcutInternal( KShortcut( cut ));
+        else
+            setShortcutInternal( KShortcut());
+        return;
+        }
+    QValueList< KShortcut > keys;
+    QStringList groups = QStringList::split( ' ', cut );
+    for( QStringList::ConstIterator it = groups.begin();
+         it != groups.end();
+         ++it )
+        {
+        QRegExp reg( "(.*\\+)\\((.*)\\)" );
+        if( reg.search( *it ) > -1 )
+            {
+            QString base = reg.cap( 1 );
+            QString list = reg.cap( 2 );
+            for( unsigned int i = 0;
+                 i < list.length();
+                 ++i )
+                {
+                KShortcut c( base + list[ i ] );
+                if( !c.isNull())
+                    keys.append( c );
+                }
+            }
+        }
+    for( QValueList< KShortcut >::ConstIterator it = keys.begin();
+         it != keys.end();
+         ++it )
+        {
+        if( _shortcut == *it ) // current one is in the list
+            return;
+        }
+    for( QValueList< KShortcut >::ConstIterator it = keys.begin();
+         it != keys.end();
+         ++it )
+        {
+        if( workspace()->shortcutAvailable( *it, this ))
+            {
+            setShortcutInternal( *it );
+            return;
+            }
+        }
+    setShortcutInternal( KShortcut());
+    }
+
+void Client::setShortcutInternal( const KShortcut& cut )
+    {
+    if( _shortcut == cut )
+        return;
+    _shortcut = cut;
+    updateCaption();
+    workspace()->clientShortcutUpdated( this );
+    }
+
+bool Workspace::shortcutAvailable( const KShortcut& cut, Client* ignore ) const
+    {
+    // TODO check global shortcuts etc.
+    for( ClientList::ConstIterator it = clients.begin();
+         it != clients.end();
+         ++it )
+        {
+        if( (*it) != ignore && (*it)->shortcut() == cut )
+            return false;    
+        }
+    return true;
     }
 
 } // namespace
