@@ -41,6 +41,7 @@
 #include <qapplication.h>
 #include <qclipboard.h>
 #include <qfile.h>
+#include <qguardedptr.h>
 #include <assert.h>
 #include <qvaluevector.h>
 
@@ -52,7 +53,7 @@ public:
     KToggleAction *aEnormousIcons;
     KToggleAction *aSmallMediumIcons;
     QValueVector<int> iconSize;
-
+            
     KDirLister* dirLister;
     bool dirSizeDirty;
 
@@ -229,6 +230,7 @@ KonqDirPart::~KonqDirPart()
     // Close the find part with us
     delete m_findPart;
     delete d;
+    d = 0;
 }
 
 void KonqDirPart::adjustIconSizes()
@@ -279,22 +281,30 @@ void KonqDirPart::slotBackgroundSettings()
 {
     QColor bgndColor = m_pProps->bgColor( widget() );
     QColor defaultColor = KGlobalSettings::baseColor();
-    KonqBgndDialog dlg( widget(), m_pProps->bgPixmapFile(), bgndColor, defaultColor );
-    if ( dlg.exec() == KonqBgndDialog::Accepted )
+    // dlg must be created on the heap as widget() can get deleted while dlg.exec(),
+    // trying to delete dlg as its child then (#124210) - Frank Osterfeld
+    QGuardedPtr<KonqBgndDialog> dlg = new KonqBgndDialog( widget(), 
+                                              m_pProps->bgPixmapFile(),
+                                              bgndColor,
+                                              defaultColor );
+    
+    if ( dlg->exec() == KonqBgndDialog::Accepted )
     {
-        if ( dlg.color().isValid() )
+        if ( dlg->color().isValid() )
         {
-            m_pProps->setBgColor( dlg.color() );
+            m_pProps->setBgColor( dlg->color() );
         m_pProps->setBgPixmapFile( "" );
     }
         else
     {
             m_pProps->setBgColor( defaultColor );
-        m_pProps->setBgPixmapFile( dlg.pixmapFile() );
+        m_pProps->setBgPixmapFile( dlg->pixmapFile() );
         }
         m_pProps->applyColors( scrollWidget()->viewport() );
         scrollWidget()->viewport()->repaint();
     }
+    
+    delete dlg;
 }
 
 void KonqDirPart::lmbClicked( KFileItem * fileItem )
@@ -318,9 +328,6 @@ void KonqDirPart::lmbClicked( KFileItem * fileItem )
         args.serviceType = fileItem->mimetype();
     args.trustedSource = true;
 
-    if ( fileItem->isLink() && fileItem->isLocalFile() ) // see KFileItem::run
-        url = KURL( url, KURL::encode_string( fileItem->linkDest() ) );
-
     if (KonqFMSettings::settings()->alwaysNewWin() && fileItem->isDir()) {
         //args.frameName = "_blank"; // open new window
         // We tried the other option, passing the path as framename so that
@@ -700,7 +707,7 @@ void KonqDirPartBrowserExtension::saveState( QDataStream &stream )
     m_dirPart->saveState( stream );
     bool hasFindPart = m_dirPart->findPart();
     stream << hasFindPart;
-    assert( ! ( hasFindPart && m_dirPart->className() == "KFindPart" ) );
+    assert( ! ( hasFindPart && !strcmp(m_dirPart->className(), "KFindPart") ) );
     if ( !hasFindPart )
         KParts::BrowserExtension::saveState( stream );
     else {
@@ -713,7 +720,7 @@ void KonqDirPartBrowserExtension::restoreState( QDataStream &stream )
     m_dirPart->restoreState( stream );
     bool hasFindPart;
     stream >> hasFindPart;
-    assert( ! ( hasFindPart && m_dirPart->className() == "KFindPart" ) );
+    assert( ! ( hasFindPart && !strcmp(m_dirPart->className(), "KFindPart") ) );
     if ( !hasFindPart )
         // This calls openURL, that's why we don't want to call it in case of a find part
         KParts::BrowserExtension::restoreState( stream );
